ADOBEDMPNS.Viewer.analytics = function ()
{

/* consts */

var EVENT_TYPE = "analytics";

var kContentViewAnalyticsEvent = "kContentViewAnalyticsEvent";
var kOverlayStartedAnalyticsEvent = "kOverlayStartedAnalyticsEvent";
var kOverlayStoppedAnalyticsEvent = "kOverlayStoppedAnalyticsEvent";
var kFolioPurchaseStartedAnalyticsEvent = "kFolioPurchaseStartedAnalyticsEvent";
var kFolioPurchaseCompletedAnalyticsEvent = "kFolioPurchaseCompletedAnalyticsEvent";

var ALLOWED_VARIABLES = [
	"stackId", // 5
	"screenId", // 6
	"stackTitle", // 7, 15, 17
	"overlayId", // 8
	"overlayType", // 9
	"isOnline", // 10
	"folioManifestId", // 14
	"isAdView",
	"productId",
	"entitlementCategory",
	"purchasePrice",
	"purchaseUnits",
    "isAutoStarted",
	"customVariable1", // 46
	"customVariable2", // 47
	"customVariable3", // 48
	"customVariable4", // 49
	"customVariable5", // 50
    "customVariable6", // 46
    "customVariable7", // 47
    "customVariable8", // 48
    "customVariable9", // 49
    "customVariable10" // 50
];

var ALLOWED_EVENTS = [
	"customEvent1", // 46
	"customEvent2", // 47
	"customEvent3", // 48
	"customEvent4", // 49
	"customEvent5", // 50
    "customEvent6", // 46
    "customEvent7", // 47
    "customEvent8", // 48
    "customEvent9", // 49
    "customEvent10" // 50
];

var CATEGORIES = {
	subscription: "subscription",
	single: "single",
	free: "free",
	external: "external"
};

/* private vars */

var variableAliases = {
	stackIndex: "stackId",
	stackId: "stackId",
	screenIndex: "screenId",
	screenId: "screenId",
	adTitle: "stackTitle",
	articleTitle: "stackTitle",
	stackTitle: "stackTitle",
	overlayId: "overlayId",
	overlayType: "overlayType",
	isOnline: "isOnline",
	issueName: "folioManifestId",
	folioManifestId: "folioManifestId",
	isAdView: "isAdView",
	productId: "productId",
	entitlementCategory: "entitlementCategory",
	purchasePrice: "purchasePrice",
	purchaseUnits: "purchaseUnits",
    isAutoStarted: "isAutoStarted",
	customVariable1: "customVariable1",
	customVariable2: "customVariable2",
	customVariable3: "customVariable3",
	customVariable4: "customVariable4",
	customVariable5: "customVariable5",
    customVariable6: "customVariable6",
    customVariable7: "customVariable7",
    customVariable8: "customVariable8",
    customVariable9: "customVariable9",
    customVariable10: "customVariable10"
};

var eventAliases = {
	customEvent1: "customEvent1", // 71
	customEvent2: "customEvent2", // 72
	customEvent3: "customEvent3", // 73
	customEvent4: "customEvent4", // 74
	customEvent5: "customEvent5", // 75
    customEvent6: "customEvent6", // 71
    customEvent7: "customEvent7", // 72
    customEvent8: "customEvent8", // 73
    customEvent9: "customEvent9", // 74
    customEvent10: "customEvent10" // 75
    
};

/* public vars */

var variables = {};

/* private methods */

var cloneAndFilter = function (obj, objCopy) {
	if (!objCopy) {
		objCopy = {};
	}
	for (var p in obj) {
		var key = variableAliases[p];
		if (ALLOWED_VARIABLES.indexOf(key) != -1) {
			// coerce all non-null values to String
			objCopy[key] = obj[p] !== null ? obj[p] + "" : null;
		}
	}
	return objCopy;
};

var send = function (eventType, eventName, overriddenVariables) {
	var variablesToSend = {};
	cloneAndFilter(variables, variablesToSend);
	cloneAndFilter(overriddenVariables, variablesToSend);
	ADOBEDMPNS.Viewer.bridge.send({
		event: {
			type: EVENT_TYPE,
			analyticsEvent: eventType,
			analyticsEventName: eventName,
			variables: variablesToSend
		}
	});
};

/* public methods */

var setCustomVariableAlias = function (realName, alias) {
	variableAliases[alias] = realName;
};

var setCustomEventAlias = function (realName, alias) {
	eventAliases[alias] = realName;
};

var trackContentView = function (articleTitle, variables) {
	var variablesCopy = cloneAndFilter(variables);
	variablesCopy[variableAliases.articleTitle] = articleTitle;
	send(kContentViewAnalyticsEvent, "", variablesCopy);
};

var trackCustomEvent = function (eventName, variables) {
	if (ALLOWED_EVENTS.indexOf(eventAliases[eventName]) != -1) {
		send(eventAliases[eventName], eventName, variables);
	}
};

// HTML & app implementations should merge overlay start & video/audio start into single hit (currently separate hits)
var trackOverlayStart = function (overlayId, overlayType, variables) {
	var variablesCopy = cloneAndFilter(variables);
	variablesCopy[variableAliases.overlayId] = overlayId;
	variablesCopy[variableAliases.overlayType] = overlayType;
	send(kOverlayStartedAnalyticsEvent, "", variablesCopy);
};

// HTML & app implementations should merge overlay stop & video/audio stop into single hit (currently separate hits)
var trackOverlayStop =  function (overlayId, overlayType, variables) {
	var variablesCopy = cloneAndFilter(variables);
	variablesCopy[variableAliases.overlayId] = overlayId;
	variablesCopy[variableAliases.overlayType] = overlayType;
	send(kOverlayStoppedAnalyticsEvent, "", variablesCopy);
};

var trackPurchaseStart = function (category, productId, variables) {
	// broker is filled in automagically
	var variablesCopy = cloneAndFilter(variables);
	variablesCopy[variableAliases.entitlementCategory] = category;
	variablesCopy[variableAliases.productId] = productId;
	send(kFolioPurchaseStartedAnalyticsEvent, "", variablesCopy);
};

var trackPurchaseComplete = function (category, productId, units, price, variables) {
	// broker is filled in automagically
	var variablesCopy = cloneAndFilter(variables);
	variablesCopy[variableAliases.entitlementCategory] = category;
	variablesCopy[variableAliases.productId] = productId;
	variablesCopy[variableAliases.purchasePrice] = price;
	variablesCopy[variableAliases.purchaseUnits] = units;
	send(kFolioPurchaseStartedAnalyticsEvent, "", variablesCopy);
};


// return public

return {
	// consts
	predefined: {
		variables: variableAliases,
		events: eventAliases,
		categories: CATEGORIES
	},
	// variables
	variables: variables,
	// methods
	setCustomVariableAlias: setCustomVariableAlias,
	setCustomEventAlias: setCustomEventAlias,
	trackContentView: trackContentView,
	trackOverlayStart: trackOverlayStart,
	trackOverlayStop: trackOverlayStop,
	trackPurchaseStart: trackPurchaseStart,
	trackPurchaseComplete: trackPurchaseComplete,
	trackCustomEvent: trackCustomEvent
};

}();

window.analyticsBridgeReady = true;
if (window.onanalyticsbridgeready) {
	window.onanalyticsbridgeready();
}
