var ADOBEDMPNS = ADOBEDMPNS ? ADOBEDMPNS : {};
ADOBEDMPNS.Utilities = ADOBEDMPNS.Utilities ? ADOBEDMPNS.Utilities : {};

ADOBEDMPNS.Utilities.convertPositionOffset = function(offset) 
{
    return offset;
};
    
ADOBEDMPNS.Utilities.getSelectorStringForElement = function(element) 
{
    var selectorString = "";
    
    var curElement = element;
    while (curElement && curElement != window.document) {
        var tag = curElement.tagName.toLowerCase();
        var className = curElement.className;
        var id = curElement.id;
        
        // We found a unique id so lets stop here and key off this
        if (id && id !== "") {
            selectorString = tag + '[id="' + id + '"]' + selectorString;
            break;
        }
        
        var orderedSiblings = $adobe.$(curElement.parentNode).find(tag);
        var indexSelector = (orderedSiblings.length > 1) ? ":eq(" + orderedSiblings.index(curElement) + ")" : "";
        
        selectorString = " " + tag + indexSelector + selectorString;
        curElement = curElement.parentNode;
    }
    
    return selectorString;
};
    
ADOBEDMPNS.Utilities.getDOMElementAtYPosition = function(yPosition)
{    
    var closestElement = null;
    var minDistance = Number.MAX_VALUE;
    $adobe.$("body *").each(function() {
                                            
                       this.offsetFromDocumentTop = ADOBEDMPNS.Utilities.convertPositionOffset($adobe.$(this).offset().top);
                       
                        // Sometimes it thinks that the same element is positioned EVERYWHERE, so lets just take our chances
                        // and not accept any node that has the exact position we are looking for. This is a hack but it works
                        // for the most part. 
                        if (this.offsetFromDocumentTop === yPosition) {
                            return;
                        }
                            
                       var curNodeBot = this.offsetFromDocumentTop + this.offsetHeight;
                                              
                       if (yPosition < this.offsetFromDocumentTop || yPosition > curNodeBot ) {
                           return;
                       }
                       
                       var curDistance = yPosition - this.offsetFromDocumentTop;
                       if (curDistance < minDistance) {
                           minDistance = curDistance;
                           closestElement = this;
                       }
                    });
    
    // Return the element's selectorString, the percentage through the element, and the first 25 characters of the innerHTML.
    var ret = {};
    if (closestElement) {
        ret.selectorString = ADOBEDMPNS.Utilities.getSelectorStringForElement(closestElement);
        ret.htmlString = closestElement.innerHTML.substr(0,25);
        
        var pixelsThrough = yPosition-closestElement.offsetFromDocumentTop;
        ret.percentageViewed = (closestElement.offsetHeight < 1) ? 0.0 : pixelsThrough / closestElement.offsetHeight;
    }
    
    return JSON.stringify(ret);
};
    
ADOBEDMPNS.Utilities.getYPositionFromElementInfo = function(elementInfo)
{
    var elements = $adobe.$(elementInfo.selectorString);
    
    if (!elements || elements.length == 0) {
        elements = $adobe.$("body *");
    }
    
    // Filter by innerHTML if ambiguous
    if (elements.length > 1) {
        elements = elements.filter(function (index) {
                                   return this.innerHTML.indexOf(elementInfo.htmlString) == 0;
                                   });
    }
    
    // Now filter by proximity to the scaled old location TODO
    
    if (!elements || elements.length == 0) {
        return 0.0;
    }
    
    var element = elements[0];
    var nodeTop = ADOBEDMPNS.Utilities.convertPositionOffset($adobe.$(element).offset().top);
    var position = nodeTop + elementInfo.percentageViewed * element.offsetHeight;
    
    return position;
};
    
ADOBEDMPNS.Utilities.getYPositionOfElementById = function(elementId)
{
    // Function requires the elementId is percent-escaped by caller because per HTML5 specs
    // all special characters (except whitespace) are valid in an element Id.
    // But when calling the jQuery search function, elementId must be backslash-escaped.
    var backslashEscapedElementId = unescape(elementId).replace(/(["'\\!*();:@&=+$,/?%[\]~.|^`{}<>])/g, "\\$1");
    var selectString = "a[name=" + backslashEscapedElementId + "], #" + backslashEscapedElementId;
    var element = $adobe.$(selectString);
    if (!element || element.length == 0) {
        return 0.0;
    }
    
    var position = element.offset();
    return ADOBEDMPNS.Utilities.convertPositionOffset(position.top);
};
    
ADOBEDMPNS.Utilities.stopAudioAndVideo = function()
{
    $adobe.$('audio, video').each(function(index) {
        this.autoplay = false;
        this.load();
    });
};
