var ADOBEDMPNS = ADOBEDMPNS ? ADOBEDMPNS : {};
ADOBEDMPNS.Viewer = ADOBEDMPNS.Viewer ? ADOBEDMPNS.Viewer : {};

//------------------------------------------------------------------
// The bridge object that actually sends messages to the Obj-C code
//------------------------------------------------------------------

ADOBEDMPNS.Viewer.bridge = 
{
    IS_RELEASE_BUILD : true,
    
    sendMessage : function(scheme, body)
    {
        var iFrame = document.createElement("iframe");
        iFrame.setAttribute("src", scheme + "://" + body);
        document.body.appendChild(iFrame); 
        iFrame.parentNode.removeChild(iFrame);
        iFrame = null;
    },
    
    send : function(data)
    {
        ADOBEDMPNS.Viewer.bridge.sendMessage("bridge", JSON.stringify(data));
    },
    
    trace : function(trace)
    {
        if (ADOBEDMPNS.Viewer.bridge.IS_RELEASE_BUILD === true) {
            return;
        }
        
        ADOBEDMPNS.Viewer.bridge.send({
                                         event:
                                         {
                                           type  : "trace", 
                                           trace : trace
                                         }
                                      });
    }
};

ADOBEDMPNS.Viewer.suppressEvent = function(event) {
    event.adobeSuppress = true;
};

//------------------------------------------------------------------
// Legacy event notification which doesn't use the bridge protocol
//------------------------------------------------------------------
ADOBEDMPNS.Viewer.notifyOfClickEvent = function(event) 
{
    ADOBEDMPNS.Viewer.suppressEvent(event);
    var element = event.target;
    if (element.href !== undefined && element.href !== null)
    {
        ADOBEDMPNS.Viewer.bridge.sendMessage("click", encodeURIComponent(element.id) + "?href=" + encodeURIComponent(element.href));
    }
};

TapDetector = function()
{
	this.startX = 0;
	this.startY = 0;
	
	var self = this;
	this.touchStartHandler = function(event){self.handleTouchStart(event)};
	this.touchMoveHandler = function(event){self.handleTouchMove(event)};
	this.touchEndHandler = function(event){self.handleTouchEnd(event)};
};

TapDetector.TAP_THRESHOLD = 10;

TapDetector.prototype.enable = function() 
{
    ADOBEDMPNS.Viewer.bridge.trace("ENABLING TAP DETECTOR");
    document.addEventListener("touchstart", this.touchStartHandler);
};

TapDetector.prototype.disable = function() 
{
    document.removeEventListener("touchstart", this.touchStartHandler);
};

TapDetector.prototype.handleTouchStart = function(event) 
{
    ADOBEDMPNS.Viewer.bridge.trace("TOUCH START " + event.adobeSuppress);
	if( event.adobeSuppress )
	{
		return;
	}
	
    document.addEventListener("touchmove", this.touchMoveHandler);
    document.addEventListener("touchend", this.touchEndHandler);
	
	this.startX = event.touches[0].screenX;
	this.startY = event.touches[0].screenY;
};

TapDetector.prototype.handleTouchMove = function(event) 
{
	if( !this.isWithinTapThreshold( event.touches[0].screenX, event.touches[0].screenY ) )
	{
		this.removeTouchMoveAndTouchEndHandlers();
	}
};

TapDetector.prototype.handleTouchEnd = function(event)
{
    ADOBEDMPNS.Viewer.bridge.trace("TOUCH END " + event.adobeSuppress);
	this.removeTouchMoveAndTouchEndHandlers();
	if( event.adobeSuppress )
	{
		return;
	}
	
    ADOBEDMPNS.Viewer.bridge.trace("SENDING TAP");
	ADOBEDMPNS.Viewer.bridge.send({event:{type:"unhandledTap",}});
};

TapDetector.prototype.removeTouchMoveAndTouchEndHandlers = function()
{
    document.removeEventListener("touchmove", this.touchMoveHandler);
    document.removeEventListener("touchend", this.touchEndHandler);
};

TapDetector.prototype.isWithinTapThreshold = function(x, y)
{
	var dx = x - this.startX;
	var dy = y - this.startY;
	
	return Math.sqrt( dx * dx + dy * dy ) < TapDetector.TAP_THRESHOLD;
};

ADOBEDMPNS.Viewer.setupTapDetector = function() {
    var tapDetector = new TapDetector();
    tapDetector.enable();
};