(function() {
    
    var EVENT_TYPE = "overlay"; 
    
    /**
     * The function to call after all the preparations for going fullscreen have been made
     * 
     * @param x The horizontal offset of the inline webview (in pixels, from the left)
     * @param y The vertical offset of the inline webview (in pixels, from the top)
     * @param width The width of the inline webview (in pixels)
     * @param height The height of the inline webview  (in pixels)
     */
    var prepareFullscreenFunction;
    
    /**
     * The function to call after we have gone fullscreen
     */
    var goFullscreenFunction;
    
    /**
     * The function to call after all the preparations for going inline have been made
     * 
     * @param x The horizontal offset of the inline webview (in pixels, from the left)
     * @param y The vertical offset of the inline webview (in pixels, from the top)
     * @param width The width of the inline webview (in pixels)
     * @param height The height of the inline webview  (in pixels)
     */
    var prepareInlineFunction;
    
    /**
     * The function to call after we have gone inline
     */
    var goInlineFunction;
    ADOBEDMPNS.Viewer.overlay = {
        
        // called from within the webview
        prepareFullscreen: function(callback) {
            prepareFullscreenFunction = callback;
            ADOBEDMPNS.Viewer.bridge.send({
                event: {
                    type: EVENT_TYPE,
                    message: "prepareFullscreen"
                }
            });
        },
        
        // called by the viewer
        prepareFullscreenReady: function(x, y, width, height) {
            if (prepareFullscreenFunction) {
                prepareFullscreenFunction({
                    x: x,
                    y: y,
                    width: width,
                    height: height
                });
            }
        },
        
        // called from within the webview
        goFullscreen: function(callback) {
            goFullscreenFunction = callback;
            ADOBEDMPNS.Viewer.bridge.send({
                event: {
                    type: EVENT_TYPE,
                    message: "goFullscreen"
                }
            });
        },
        
        // called by the viewer
        fullscreenReady: function() {
            if (goFullscreenFunction) {
                goFullscreenFunction();
            }
        },
        
        // called from within the webview
        prepareInline: function(callback) {
            prepareInlineFunction = callback;
            ADOBEDMPNS.Viewer.bridge.send({
                event:
                {
                    type: EVENT_TYPE,
                    message: "prepareInline"
                }
            });
        },
        
        // called by the viewer
        prepareInlineReady: function(x, y, width, height) {
            if (prepareInlineFunction) {
                prepareInlineFunction({
                    x: x,
                    y: y,
                    width: width,
                    height: height
                });
            }
        },
        
        // called from within the webview
        goInline: function(callback) {
            goInlineFunction = callback;
            ADOBEDMPNS.Viewer.bridge.send({
                event: {
                    type: EVENT_TYPE,
                    message: "goInline"
                }
            });
        },
        // called by the viewer
        inlineReady: function() {
            if (goInlineFunction) {
                goInlineFunction();
            }
        }
    };
    
    if (!window.DeviceOrientationEvent) {
		window.onorientationchange = function () {
			var e = document.createEvent('Events');
			e.initEvent('orientationchange', true, false);
			document.dispatchEvent(e);
		}	
	};
    
    window.overlayBridgeReady  = true;
    if (window.onoverlaybridgeready) {
        window.onoverlaybridgeready();
    }
    
})();
