/**
 * The main application file.
 */
var ADOBE = ADOBE || {};

ADOBE.AppView = Backbone.View.extend({
	el: $("body"),

	// Stores the FolioItemView instances.
	folioItemViewArray: [],
	
	// Displays the grid of folios.
	$grid: null,
	
	// Collection of folios.
	libraryCollection: null,
	
	// The HTML for the subscribe buttons.
	subscriptions: "",
	
	isOnline: false,
	
	showMore: null,
	
	folios: null,
	folios2: null,
	folios3: null,
	
	// The number of folios to add for each page.
	foliosPerPage: 12,
	
	subscribeDialog: null,
	
	// Whether or not a subscription is active.
	isSubscriptionActive: false,
	
	// Whether or not the user owns the most recent folio.
	userOwnsLatestFolio: false,

	LBL_SIGN_OUT: "Sign Out",
	LBL_SIGN_IN: "Sign In",
	LBL_RESTORE_ALL_PURCHASES: "Restore All Purchases",
	LBL_REMOVE_ISSUES_FROM_IPAD: "Remove Issues from iPad",
	
	initialize: function(isAPIAvailable, isOnline) {
		// Set a flag for the API availability in the ADOBE namespace.
		ADOBE.isAPIAvailable = isAPIAvailable;
		
		this.isOnline = this.check_online();

		if (isAPIAvailable) {
			var transaction = adobeDPS.libraryService.updateLibrary();
			transaction.completedSignal.addOnce(this.updateLibraryHandler, this);
		} else { // Testing on the desktop.
			this.updateLibraryHandler();
		}
		
		// This checks the device every X number of seconds to see if the network state has changed.
		var networkState;
		var networkStateCtr = 0;
		setInterval(function(){		
			tempNetResult = ADOBE.AppView.prototype.check_online();
			if (tempNetResult != networkState){
				networkState = tempNetResult;
				if (networkStateCtr>0){
					location.reload();
				}
			}			
			networkStateCtr = networkStateCtr + 1;
			
		},8000);
	},
	
	updateLibraryHandler: function() {		
		var loginLbl;
		if (ADOBE.isAPIAvailable) {
			// Put the FolioStates in the ADOBE namespace for easier lookup later.
			ADOBE.FolioStates = adobeDPS.libraryService.folioStates;
			
			this.folios = []; // This is an array of all folios available to the user.
			this.folios2 = []; // This is an array of all folios that the user is entitled to.
			this.folios3 = []; // This is an array of only folios the user has downloaded.
			
			// Sort the folios descending.
			var list = adobeDPS.libraryService.folioMap.sort(function (a, b) {
				if (a.publicationDate < b.publicationDate)
					return 1;
				else if (a.publicationDate > b.publicationDate)
					return -1;
				else
					if (a.folioNumber < b.folioNumber)
						return 1;
					else if (a.folioNumber > b.folioNumber)
						return -1;
					else
						return 0;
			});
	 		
	 		// list is an associative array so put them in a regular array.
			// Modified to only display folios the user is entitled to.
			for (var i in list) {
				var folio = list[i];
				if (folio.state >= 200){
					this.folios2.push(folio);
				}
				this.folios.push(folio);
			}
			
			// Also build a list of folios that the user has downloaded.
			for (var i in list) {
				var folio = list[i];
				if (folio.state >= 400){
					this.folios3.push(folio);
				}
			}
			
			// If the latest folio is not purchasable then the user is entitled to it.
			// If true then do not display the subscription button.
			if (this.folios.length > 0) {
				var latestFolio = this.folios[0];
				this.userOwnsLatestFolio = !(latestFolio.state == ADOBE.FolioStates.PURCHASABLE || latestFolio.state == ADOBE.FolioStates.UNAVAILABLE || latestFolio.state == ADOBE.FolioStates.INVALID);
			} else if (!this.isOnline) { // Folio list is empty and user is not online.
				alert("Unable to connect to the internet.");
			}

			if (!this.userOwnsLatestFolio) {
				// Loop through the subscriptions and populate the buttons.
				var availableSubscriptions = adobeDPS.receiptService.availableSubscriptions;
				for (var s in availableSubscriptions) {
					var availableSubscription = availableSubscriptions[s];
					if (availableSubscription.isActive()) { // Users owns a subscription so do not display the subscription menu option. 
						this.isSubscriptionActive = true;
						break;
					} else { // Create a string for the subscription buttons.
						this.subscriptions += "<div class='subscribe-button' id='" + availableSubscription.productId + "'>" + availableSubscription.duration + " subscription for " + availableSubscription.price + "</div>";
						this.isSubscriptionActive = false;
					}
				}
			}
			
			// Determine the login label for the drop down menu.
			loginLbl = adobeDPS.authenticationService.isUserAuthenticated ? this.LBL_SIGN_OUT: this.LBL_SIGN_IN;
		} else { // Placeholder values for testing on the desktop.
			this.subscriptions += "<div class='subscribe-button' id='1year'>1 Year Subscription for $19.99</div>";
			this.subscriptions += "<div class='subscribe-button' id='1month'>1 Month Subscription for $1.99</div>";
			loginLbl = this.LBL_SIGN_IN;
		}
		
		var html = "";

		// Swap in a different banner depending if we're online or not.
		if (ADOBE.AppView.prototype.check_online()){
			html += "<div id='banner'>";
			html += 	"<div id='banner-logo'></div>";
			html += 	"<span class='banner-text'></span>";
			html += "</div>";
		}
		else{
			html += "<div class='offline-runner'>Your device is not connected to the Internet. You can only view issues that have already been downloaded to your device.</div>";
			html += "<div id='banner'>";
			html += 	"<span class='banner-text'>AUGUST 2013 LIONS</span>";
			html += "</div>";
		}

		html += "<div id='header'>";
			
		// If API is not available then testing on the desktop so show the button, otherwise only if this is an entitlement viewer.
		if (!ADOBE.isAPIAvailable || ADOBE.Config.IS_ENTITLEMENT_VIEWER)
			html +=     "<div id='print-subscriber-login'>" + loginLbl + "</div>";

			// If API is not available then testing on the desktop so show the button, otherwise only if subscriptions are available.
			html +=     "<div id='subscribe'>Subscribe</div>";

			html +=		"<div id='magtitle'>NATIONAL GEOGRAPHIC MAGAZINE</div>";
			
			var index = ADOBE.Config.IS_ONE_UP_VIEW_DEFAULT ? 0 : 1;
			html +=     "<div class='navbar' id='navbar' default-selected-index='" + index + "'>";
			//html +=         "<div off-skin-style='view-toggle-big-off' on-skin-style='view-toggle-big-on'></div>";
			//html +=         "<div off-skin-style='view-toggle-grid-off' on-skin-style='view-toggle-grid-on'></div>";
			html +=     "</div>";

			html +=     "<div class='drop-down' id='header-drop-down'>";
			html +=         "<div id='restore-all-purchases'>" + this.LBL_RESTORE_ALL_PURCHASES + "</div>";

		// If testing on desktop then include the switch otherwise make sure it is supported.
		if (!ADOBE.isAPIAvailable || adobeDPS.settingsService.autoArchive.isSupported){
			html +=     	"<div id='auto-archive' class='flip-switch' state='" + (!ADOBE.isAPIAvailable || adobeDPS.settingsService.autoArchive.isEnabled ? "on" : "off") + "'>Auto Archive</div>";
		}
			html +=     "</div>";
		    html += "</div>";
		    
		    // Reload bar. Comment out when not error handling.
			//html += "<div style='height: 20px; background-color: red;' onclick='location.reload()'></div>";

		    // The container to hold the grid of folios.
		    html += "<div id='gridheader'></div>";
		    html += "<div id='bghider'></div>";
		    html += "<div id='grid'>";
			html += 	"<div id='subscription-tile'></div>";
			html += 	"<div id='loading'>Loading...</div>";
		    html += "</div>"
		    html += "<div id='show-more'>Show More <img src='adobeimg/icon_down_arrow.png'></div>"


		// Uncomment the textarea below to enable debug output via debug().
		//html += "<textarea class='debug'></textarea>";
		window.debug = function(value) {
			$(".debug").val($(".debug").val() + ($(".debug").val() == "" ? "" : "\n") + value);
		}
		
		$("body").html(html);
		$(".offline-runner").fadeIn("slow");
		
		this.$grid = $("#grid");
		
		if (ADOBE.AppView.prototype.check_online()){
		
			// Pull in a random banner.
			var feedIndex;
		   	var totallen = issueData.length;
		   	var maxHeaders = issueData[(totallen-1)].headers.headerimg.length;
		   	intRandomVal = Math.floor(Math.random() * (maxHeaders)) + 1;
			strBannerURL = issueData[(totallen-1)].headers.headerimg[(intRandomVal-1)];
			strBannerText = issueData[(totallen-1)].headers.headertxt[(intRandomVal-1)];
			
			
			$('#banner').css("background-image","url(" + strBannerURL + ")");
			$('.banner-text').append(strBannerText);
		}
		
		if (ADOBE.isAPIAvailable && (this.isSubscriptionActive || this.userOwnsLatestFolio)){
			$("#subscribe").css("display", "none");
			$("#subscription-tile").css("display", "none");
		}
		
		$("#header-drop-down").dropDown({verticalGap: -12});
		
		this.showMore = $("#show-more");
		
		var scope = this;
		
		// Handler for the auto archive switch in the drop down.
		$("body").on("change", "#auto-archive", function(e, isOn){ scope.autoArchive_changeHandler(e, isOn) });

		// Handler for the drop down menu.
		$("body").on("change", "#header-drop-down", function(e){ scope.header_dropDownChangeHandler(e) });
		
		// Click handler for "show more" link at the bottom of the grid.
		this.showMore.on("click", function(){ scope.addFolios() });
		
		// Click handler for the login dialog.
		$("#print-subscriber-login").on("click", function(){ scope.displayLoginDialog() });
		
		$(window).on("resize", function(){ scope.setGridHeight() });

		$("#subscribe").on("click", function(){ scope.displaySubscribeDialog() });
		$("body").on("subscribeButtonClicked", function(){ scope.displaySubscribeDialog() });
		
		$("body").on("folioThumbClicked", function(e, folio, elementId){ scope.displayPreviewDialog(e, folio, elementId) });
		
		$("body").on("displaySectionsClicked", function(e, folio){ scope.displaySectionsView(folio) });

		// Triggered from the dialog when a purchase is successful.
		$("#subscribe, #subscription-tile").on("click", function() {
			var subscribeDialog = new ADOBE.SubscribeDialog({model: scope.subscriptions});
			$("body").append(subscribeDialog.render().el);
			subscribeDialog.open();
		
			// Triggered from the dialog when a purchase is successful.
			$("body").on("subscriptionPurchased", function() {
				// Remove the subscribe button.
				$("#subscribe").remove("display", "none");
				$("#subscription-tile").remove("display", "none");
				
				$("body").off("subscriptionPurchased");
			});
		});
		

		
		// This performs the continuous scrolling auto-loading.
		setTimeout(function(){
			$(window).scroll(function(){
			  if($(window).scrollTop() + 50 > $(document).height() - $(window).height() ){
				scope.addFolios();
			  }
			});
  		},1500);
		
		if (ADOBE.isAPIAvailable) {
			// The collection creates a clone of the folio objects so addFolios() passes a reference to the object.
			// Since the folios are not on a server we don't need to load anything so pass the folios to the constructor.
			
			var folioData;
			if (ADOBE.AppView.prototype.check_online()){
				folioData = this.folios2;
			}
			else{
				folioData = this.folios3;
			}
			
			this.libraryCollection = new ADOBE.LibraryCollection(folioData);
			
			// Add the folios which are currently available. On the first launch this
			// does not guarentee that all folios are immediately available. The callback
			// below for folioMap.addedSignal will handle folios which are added after
			// startup. Added does not mean, pushed from folio producer, rather they
			// are folios that the viewer becomes aware of after startup.
			this.addFolios();
			
			// Add a listener for when new folios are added.
			adobeDPS.libraryService.folioMap.addedSignal.add(function(folios) {
				for (var i = 0; i < folios.length; i++) {
					scope.addFolio(folios[i]);
				}
			}, this);
		} else { // Testing on the desktop so load the data from the fulfillment server.
			_.bindAll(this, "addFolios");
			this.libraryCollection = new ADOBE.LibraryCollection();
			this.libraryCollection.url = "http://lighthouse.adobe.com/dps/v2_library_store_templates/fulfillment_proxy.php?accountId=1f60b887d5ce468a9279d665a62da41b";
			this.libraryCollection.on("all", this.addFolios);
			this.libraryCollection.fetch({dataType: "xml"});
		}
		
		// Add to global for the configurator to set values.
		window.Config = ADOBE.Config;
	},
	
	addFolios: function() {
		var subsDisplay = $("#subscription-tile").css("display");
 		var foliosToDisplay;
		if (subsDisplay == "none"){
			foliosToDisplay = 12;
		}
		else{
			foliosToDisplay = 13;
		}
		foliosToDisplay = 12;
		
		$("#loading").remove();
		if (this.libraryCollection.length == 0){
			$("#show-more").remove();
			$('.offline-runner').empty();
			$('.offline-runner').append('You have no downloaded issues. To download or purchase an issue, please connect to the Internet.');
			return;
		}
		
		var startIndex = this.getNumFoliosVisible();
		var endIndex = Math.min(startIndex + foliosToDisplay, this.libraryCollection.length);
		for (var i = startIndex; i < endIndex; i++) {
			// When using the DPS api this is a clone of the original folio.
			var folio = this.libraryCollection.at(i);
			
			// Testing on the desktop so create the path to the image.
			if (!ADOBE.isAPIAvailable)
				folio.attributes.libraryPreviewUrl +=  "/portrait";
				
			var view = new ADOBE.FolioItemView({model: folio});
			var el = view.render().el;
			this.$grid.append(el);
			
			this.folioItemViewArray.push(view);
		}
		
		this.setGridHeight();
	},

	getNumFoliosVisible: function() {
		return this.$grid.children().length - 1;
	},
	
	// This will be triggered when folios are added through the API.
	addFolio: function(folio) {
		$("#loading").remove();
		
		var folioData;
		if (this.check_online()){
			folioData = folios2;
		}
		else{
			folioData = folios3;
		}
		
		var len = this.folioData.length;
		// Find the insert index. Folios are sorted by publicationDate with the most recent first.
		for (var i = 0; i < len; i++) {
			if (folio.publicationDate >= this.folioData[i].publicationDate)
				break;
		}
		
		// Add the folio to the collection.
		this.libraryCollection.add(folio, {at: i});
		
		// Add the folio to the folios.
		this.folioData.splice(i, 0, folio);
		
		// Figure out if the user has or is entitled to the latest folio or has a subscription covering today's date.
		// If the latest folio is not purchasable then the user is entitled to it.
		// If true then do not display the subscription button or tile.
		if (this.folioData.length > 0) {
			var latestFolio = this.folioData[0];
			this.userOwnsLatestFolio = !(latestFolio.state == ADOBE.FolioStates.PURCHASABLE || latestFolio.state == ADOBE.FolioStates.UNAVAILABLE || latestFolio.state == ADOBE.FolioStates.INVALID);
		}

		// Figure out if this folio should be dispayed.
		// Folios can be added in any order so see if this folio is within the range of publication
		// dates of the folios that are currently displayed.
		var numFoliosDisplayed = this.getNumFoliosVisible();
		var subsDisplay = $("#subscribe").css("display");
		var foliosToDisplay;
		if (subsDisplay == "none"){
			foliosToDisplay = 12;
		}
		else{
			foliosToDisplay = 13;
		}
		var foliosToDisplay = 12;
		
		// Switched from this.foliosPerPage
		var endIndex = Math.max(foliosToDisplay, numFoliosDisplayed);
		if (i < endIndex) {
			var view;
			// See more button is visible so remove the last folio view before inserting a new one.
			if (numFoliosDisplayed >= foliosToDisplay) {
				$("#grid div.folio-item-view:last-child").remove();
				 view = this.folioItemViewArray.pop();
				 view.clear();
			}
				
			view = new ADOBE.FolioItemView({model: this.libraryCollection.at(i)});
			var el = view.render().el;
			
			if (numFoliosDisplayed == 0)
				this.$grid.append(el);
			else
				$("#grid div.folio-item-view").eq(i).before(el);
				
			this.folioItemViewArray.splice(i, 0, view);
		}
		
		// Hide the subscribe button.
		if (this.userOwnsLatestFolio){
			$("#subscribe").hide();
			$("#subscription-tile").hide();
		}
		else{
			$("#subscribe").show();
			$("#subscription-tile").show();
		}
		
		this.setGridHeight();
	},
	
	setGridHeight: function() {
		var windowWidth = $(window).width();
		// Need to explicitly set the width otherwise it doesn't always update if width=100% in css.
		$("#header").width(windowWidth);
		var numFoliosDisplayed = this.getNumFoliosVisible();
		if($("#subscribe").css("display") == "none"){
			// user is a subscriber.
			this.$grid.css("height", Math.ceil(numFoliosDisplayed / 3) * (windowWidth > $(window).height() ? 187 : 187));
		}
		else{
			// user is a subscriber.
			this.$grid.css("height", Math.ceil((numFoliosDisplayed + 2)/ 3) * (windowWidth > $(window).height() ? 187 : 187));
		}
		//this.$grid.css("height", Math.ceil(numFoliosDisplayed / 3) * (windowWidth > $(window).height() ? 188 : 188));
		this.showMore.css("display", numFoliosDisplayed < this.libraryCollection.length ? "block" : "none");
	},
	
	// Handler for the drop down menu.
	header_dropDownChangeHandler: function(e) {
		var selectedLabel = $(e.target).dropDown("getSelectedLabel");
		if (selectedLabel == this.LBL_RESTORE_ALL_PURCHASES) {	// Display the restore dialog.
			this.displayRestorePurchasesDialog();
		} else if (selectedLabel == this.LBL_REMOVE_ISSUES_FROM_IPAD) {
			var archiveView = new ADOBE.ArchiveView({model: this.libraryCollection});
			$("body").append(archiveView.render().el);
			
			// Need to remove the grid so it is not scrollable in the background.
			var previewScrollPosition = $(window).scrollTop(); // get the current scroll position
			$("#grid").hide();
			
			archiveView.$el.on("archiveViewClosed", function() {
				$(window).scrollTop(previewScrollPosition); // set the scroll position back to what it was.
				$("#grid").show();
				
				archiveView.$el.off("archiveViewClosed");
			});
		}
	},
	
	displayRestorePurchasesDialog: function() {
		var restoreDialog = new ADOBE.RestoreDialog();
		$("body").append(restoreDialog.render().el);
		restoreDialog.open();
		
		restoreDialog.$el.on("restorePurchasesStarted", function(e, transaction) {
			restoreDialog.$el.off("restorePurchasesStarted");
			
			$("#header #title, #header #title-image").hide();
			$("<div id='spinner-title'><div id='spinner' class='spinner'></div>Restoring Purchases...</div>").appendTo("#header");
			
			// Options for the indeterminate spinner.
			var opts = {
					  lines: 13, // The number of lines to draw
					  length: 3, // The length of each line
					  width: 2, // The line thickness
					  radius: 6, // The radius of the inner circle
					  corners: 0, // Corner roundness (0..1)
					  rotate: 0, // The rotation offset
					  direction: 1, // 1: clockwise, -1: counterclockwise
					  color: '#000000', // #rgb or #rrggbb
					  speed: 1, // Rounds per second
					  trail: 60, // Afterglow percentage
					  shadow: false, // Whether to render a shadow
					  hwaccel: false, // Whether to use hardware acceleration
					  className: 'spinner', // The CSS class to assign to the spinner
					  zIndex: 2e9, // The z-index (defaults to 2000000000)
					  top: -1, // Top position relative to parent in px
					  left: 80 // Left position relative to parent in px
					};
			var target = document.getElementById("spinner");
			var spinner = new Spinner(opts).spin(target);
			
			transaction.completedSignal.addOnce(function() {
				$("#header #title, #header #title-image").show();
				spinner.stop();
				$("#spinner-title").remove();
			}, this);
		});
	},
	
	displayLoginDialog: function() {
		if (!ADOBE.isAPIAvailable || !adobeDPS.authenticationService.isUserAuthenticated) {
			var loginDialog = new ADOBE.LoginDialog();
			$("body").append(loginDialog.render().el);
			
			var scope = this;
			// Triggered from the dialog when a login is successful.
			loginDialog.$el.on("loginSuccess", function() {
				$("body").append("<div class='modal-background'><div class='dialog' id='wait-dialog'>Please Wait..</div></div>");
				$("#wait-dialog").addClass("pop");
				setTimeout(function(){location.reload()},2000);
			});
		} else {	
			$("body").append("<div class='modal-background'><div class='dialog' id='wait-dialog'>Please Wait..</div></div>");
			$("#wait-dialog").addClass("pop");
			adobeDPS.authenticationService.logout();
			setTimeout(function(){location.reload()},2000);
		}
	},
	
	displaySubscribeDialog: function() {
		if (!this.subscribeDialog) {
			this.subscribeDialog = new ADOBE.SubscribeDialog({model: this.subscriptions});

			var scope = this;
			this.subscribeDialog.$el.on("subscribeDialogClosed", function() {
				scope.subscribeDialog = null;
			});
			
			$("body").append(this.subscribeDialog.render().el);
		}
	},
	
	// Handler for when a user changes the auto archive setting.
	autoArchive_changeHandler: function(e, isOn) {
		adobeDPS.settingsService.autoArchive.toggle(isOn);
	},
	
	check_online: function() {

	   var xhr = new ( window.ActiveXObject || XMLHttpRequest )( "Microsoft.XMLHTTP" );
	 
	   // Open new request as a HEAD to the root hostname with a random param to bust the cache
	   xhr.open( "HEAD", "http://ajax.googleapis.com/ajax/libs/jquery/1.11.0/jquery.min.js?rand=" + Math.floor((1 + Math.random()) * 0x10000), false );
	   
	   // Issue request and handle response
	   try {
		   xhr.send();
		   return true;
	   } catch (error) {
		 	return false;
	   }
	},
});