// Page Template Library
// managing the document as a collection of flows

// begin private scope
(function(pgt, undef) {

if( !pgt )
    alert("pgt-util.js is required");
pgt.provides("pgt-doc");
pgt.requires("pgt-style");
pgt.requires("pgt-flow");

var noForeignFlows = function(elem, depth)
{
    if( depth === 0 )
        return true;
    var pgtstyle = elem.pgtstyle;
    if( !pgtstyle )
        return true;
    var flowName = pgtstyle["flow-name"];
    var display = pgtstyle.display;
    return !flowName && display != "oeb-page-head" && display != "oeb-page-foot";
};

pgt.Doc = function(pgt)
{
    if( pgt )
        this.protectedInit(pgt);
};

pgt.createDoc = function()
{
    return new this.Doc(this);
};

var Doc_proto = pgt.Doc.prototype;

Doc_proto.protectedInit = function(pgt)
{
    this.pgt = pgt;
    this.elements = [];
    this.lookupOffset = 0;
    var self = this;
    this.itemFilter = function(item)
    { 
        return item.getSourceElement().pgtstyle.startOffset < self.lookupOffset;
    };
    this.collectFlows = function(element, isRoot)
    {
        collectFlows(self, element, isRoot);
    };
    this.getPGTProperty = function(elem, name)
    {
        if( !elem.pgtstyle /*|| elem.pgtstyle.styler != self.styler*/ )
            self.styler.ensureNode(elem);
        if( elem.pgtstyle )
            return elem.pgtstyle[name];
        return null;
    };    
};

Doc_proto.addElement = function(element)
{
    if( !this.rootElement )
        this.rootElement = element;
    else
        this.pgt.error("multiple elements not supported");
    this.elements.push(element); // TODO: remove
    this.flows = null;
};

Doc_proto.setStyle = function(style)
{
    this.style = style;
    this.flows = null;
};

Doc_proto.setViewTracker = function(tracker)
{
    this.viewTracker = tracker;
};

var collectFlows = function(doc, element, isRoot)
{
    var display = element.pgtstyle["display"];
    var oebOpts;
    var flowName = element.pgtstyle["flow-name"];
    if( !flowName )
    {
        if( display == "oeb-page-head" || display == "oeb-page-foot" )
        {
            flowName = display;
            oebOpts = "static exclusive";
        }
        else if( isRoot )
            flowName = "body";
    }
    if( flowName )
    {
        var flow = doc.flows[flowName];
        if( !flow )
        {
            flow = doc.pgt.createFlow();
            doc.flows[flowName] = flow;
            if( flowName != "body" )
                flow.setItemFilter(doc.itemFilter);
        }
        var item = doc.pgt.createFlowItem(element, element.pgtstyle.startOffset);
        var opts = element.pgtstyle["flow-options"];
        if( !opts )
            opts = oebOpts;
        if( opts )
        {
            if( opts.indexOf("exclusive") >= 0 )
                item.setOption("exclusive", true);
            if( opts.indexOf("static") >= 0 )
                item.setOption("static", true);
        }
        var pri = element.pgtstyle["flow-priority"];
        if( pri !== undef )
            item.setOption("priority", pri-0 );
        var linger = element.pgtstyle["flow-linger"];
        if( linger !== undef )
            item.setOption("linger", linger-0 );
        item.setFilter( noForeignFlows );
        item.setGetPropertyFunc( doc.getPGTProperty );
        flow.addItem( item );
    }
};

Doc_proto.resetFlows = function()
{
    var pgt = this.pgt;
    this.flows = {};
    this.lookupOffset = 0;
    this.styler = this.style.createStyler( this.rootElement, true, 0);
    this.styler.setPostStylingHook(this.collectFlows);
};

Doc_proto.getFlow = function(name)
{
    return this.flows[name];
};

Doc_proto.layoutPage = function(pageContainer)
{
    return this.style.layoutPage(pageContainer, this);
};

Doc_proto.setLookupOffset = function(offset)
{
    this.lookupOffset = offset;
    this.styler.ensureOffset(offset);
};

Doc_proto.beginPage = function()
{
    for( var name in this.flows )
    {
        var flow = this.flows[name];
        flow.beginPage(this);
    }
};

Doc_proto.endPage = function()
{
    for( var name in this.flows )
    {
        var flow = this.flows[name];
        flow.endPage();
    }
};

Doc_proto.hasContentInFlow = function(flowName)
{
    var flow = this.getFlow(flowName);
    if( flow == null )
        return false;
    var count = flow.getItemCount();
    if( count == 0 )
        return false;
    var item = flow.getItem(0);
    var element = item.getSourceElement();
    return element.pgtstyle.startOffset < this.lookupOffset;        
};

Doc_proto.getCurrentBodyOffset = function(flowName)
{
    this.styler.ensureOffset(20); // ensure that the root element is processed
    var flow = this.getFlow("body");
    if( flow == null )
        return Number.MAX_VALUE;
    var flowPos = flow.getFlowPosition();
    if( flowPos == null )
        return Number.MAX_VALUE;
    this.styler.ensureNode(flowPos.node);
    return flowPos.node.pgtstyle.startOffset + flowPos.offset;   
};

Doc_proto.hasMoreBodyContent = function()
{
    var flow = this.getFlow("body");
    if( flow == null )
        return false;
    return flow.getItemCount() > 0;
};

Doc_proto.getRootElement = function()
{
    return this.rootElement;
};

Doc_proto.addView = function(master, view, recursive, proxyTarget)
{
    var viewTracker = this.viewTracker;
    if( !viewTracker )
        return;
    var id = master.id;
    if( id )
        viewTracker.reportId(id);
    if( proxyTarget )
        viewTracker.reportProxy(master, view, proxyTarget);
    if( !recursive )
        return;
    var mc = master.firstChild;
    var vc = view.firstChild;
    while( mc )
    {
        if( mc.nodeType == 1 )
            this.addView(mc, vc, true);
        mc = mc.nextSibling;
        vc = vc.nextSibling;
    }
};

})(pgt);