// Page Template Library
// dynamic styling

// begin private scope
(function(pgt, undef) {

if( !pgt )
    alert("pgt-util.js is required");
pgt.provides("pgt-dstyle");
pgt.requires("pgt-style");
pgt.requires("pgt-expr");

pgt.DynamicStyle = function(pgt)
{
    if( pgt )
        this.protectedInit(pgt);
};

pgt.createDynamicStyle = function()
{
    return new this.DynamicStyle(this);
};

var DynamicStyle_proto = new pgt.Style();
pgt.DynamicStyle.prototype = DynamicStyle_proto;
DynamicStyle_proto.constructor = pgt.DynamicStyle;

DynamicStyle_proto.protectedInit = function(pgt)
{
    pgt.Style.prototype.protectedInit.call(this, pgt);
    this.scope = this.pgt.createExprScope();
    this.resolveQueue = [];
    this.pgtDepth = 0;
};

DynamicStyle_proto.calc = function(val)
{
    if( !this.context )
        this.pgt.error( "no context, call prepare()" );
    var res = this.context.calc(val);
    if( typeof res == "number" )
        return res + "px";
    return res;
};

DynamicStyle_proto.parseValue = function(text, propName)
{
    var r, res, expr;
    if( (r = text.match(/^expr\((.*)\)\s*$/)) !== null )
    {
        expr = this.pgt.exprParse(r[1], null, propName);
        this.resolveQueue.push(expr);
        return expr;
    }
    else if( (r = text.match(/^func\((.*)\)\s*$/)) !== null )
    {
        // for testing
        expr = this.pgt.exprParse(r[1], null, propName, true);
        this.resolveQueue.push(expr);
        return expr;
    }
    else if( text.match( this.pgt.patterns.cssLength ) && (this.pgtDepth > 0 || text.match(/r(px|em)$/)) )
        return this.pgt.exprParse(text, null, propName);
    else
        return text;
};

DynamicStyle_proto.parseAtRule = function(ruleName, ruleHead, ruleBody, cond)
{
    switch( ruleName )
    {
    case "when" :
        this.privateParseWhen(ruleHead, ruleBody, cond);
        break;
    case "define" :
        this.pgtDepth++;
        this.privateParseDefine(ruleBody, cond);
        this.pgtDepth--;
        break;
    case "pgt" :
        var saveAtOnly = this.atOnly;
        this.atOnly = false;
        this.parseStylesheet(ruleBody, cond);
        this.atOnly = saveAtOnly;
        break;
    default:
        pgt.Style.prototype.parseAtRule.call( this, ruleName, ruleHead, ruleBody, cond );
    } 
};

DynamicStyle_proto.privateMakeCondition = function(expr, cond)
{
    var self = this;
    return function(elem)
    {
        if( cond && !cond(elem) )
            return false;
        var r = self.context.calc(expr);
        if( r === false || r === 0 )
            return false;
        return true;
    };
};

DynamicStyle_proto.privateParseWhen = function(ruleHead, ruleBody, cond)
{
    var whenExpr = this.pgt.exprParse(ruleHead);
    this.resolveQueue.push(whenExpr);
    var newCond = this.privateMakeCondition(whenExpr, cond);
    this.parseStylesheet(ruleBody, newCond);
};

DynamicStyle_proto.privateParseDefine = function(ruleBody, cond)
{
    if( cond )
        this.pgt.error("no @define inside @when or @mode!");
    else
    {
        var props = this.parseProperties(ruleBody, cond);
        for( var name in props )
            this.define( name, props[name] );
    }
};

DynamicStyle_proto.define = function(name, val)
{
    this.scope.define(name, val);
};

DynamicStyle_proto.resolve = function()
{
    this.scope.resolve();
    while( this.resolveQueue.length > 0 )
    {
        var expr = this.resolveQueue.pop();
        this.pgt.exprResolve(expr, this.scope);
    }
};

DynamicStyle_proto.prepare = function(context)
{
    if( !context )
        context = this.pgt.createExprContext();   
    this.context = context;
};

})(pgt);