// Page Template Library
// CSS value expression calculator

// begin private scope
(function(pgt, undef) {

if( !pgt )
    alert("pgt-util.js is required");
pgt.provides("pgt-expr");

var calcMark = {}; // marker object

var applyTo = function(expr, params)
{
    var type = typeof expr;
    if( type == "object" && expr.pgtApplyTo !== undef )
        return expr.pgtApplyTo(params);
    else
        return expr;    
};

var Term = function(ctr)
{
    if( ctr )
        this.constructor = ctr;
};

var Term_proto = Term.prototype;

Term_proto.pgtCalc = function(context)
{
    if( this.context === context )
    {
        if( this.result === calcMark )
            this.pgt.error("Circular dependency"); // this node is being calculated
        // already calculated this node
    }
    else
    {
        // this node was not yet calculated in this context
        this.result = calcMark;
        this.context = context;
        this.result = this.pgtCalcCore(context);
    }
    return this.result;
};

Term_proto.pgtDependsOn = function(val)
{
    if( this.depVal === val )
    {
        if( this.depResult == calcMark )
            this.pgt.error("Circular dependency");
        // already calculated this node
    }
    else
    {
        // this node dependency on the value val was not yet determined
        this.depVal = val;
        this.depResult = calcMark;
        this.depResult = this.pgtDependsOnCore(val);
    }
    return this.depResult;
};

Term_proto.pgtResolve = function(scope)
{
    if( this.resolved )
        return;
    this.resolved = true;
    this.pgtResolveCore(scope);
};

Term_proto.pgtApplyTo = function(params)
{
    return this;
};

Term_proto.pgtDependsOnCore = function(val)
{
    return val === this;
};

Term_proto.pgtResolveCore = function(scope)
{
};

var NameTerm = function(pgt,name)
{
    this.pgt = pgt;
    this.name = name;
};

var NameTerm_proto = new Term(NameTerm);
NameTerm.prototype = NameTerm_proto;

NameTerm_proto.pgtCalcCore = function(context)
{
    return context.calc(this.resolvedValue);
};

NameTerm_proto.pgtDependsOnCore = function(val)
{
    return this.pgt.exprDependsOn(this.resolvedValue, val);
};

NameTerm_proto.pgtResolveCore = function(scope)
{
    this.resolvedValue = scope.resolveName(this.name);
};

var AccessTerm = function(pgt, val, name)
{
    this.pgt = pgt;
    this.val = val;
    this.name = name;
};

var AccessTerm_proto = new Term(AccessTerm);
AccessTerm.prototype = AccessTerm_proto;

AccessTerm_proto.pgtCalcCore = function(context)
{
    return context.calc(this.resolvedValue);
};

AccessTerm_proto.pgtDependsOnCore = function(val)
{
    return this.pgt.exprDependsOn(this.resolvedValue, val);
};

AccessTerm_proto.pgtResolveCore = function(scope)
{
    this.pgt.exprResolve(this.val, scope);
    this.resolvedValue = scope.resolveAccess(this.val.resolvedValue, this.name);
};

var InfixTerm = function(pgt, op, v1, v2)
{
    this.pgt = pgt;
    this.op = op;
    this.v1 = v1;
    this.v2 = v2;
    if( op == '?' && (v2.constructor != InfixTerm || v2.op != ':') )
        this.pgt.error("unexpected state");
};

var InfixTerm_proto = new Term(InfixTerm);
InfixTerm.prototype = InfixTerm_proto;

InfixTerm_proto.pgtCalcCore = function(context)
{
    var v1 = context.calc(this.v1);
    var v2 = this.v2;
    switch(this.op)
    {
    case "&&" :
        if( !v1 )
            return v1;
        break;
    case "||" :
        if( v1 )
            return v1;
        break;
    case '?' :
        if( v2.op != ':' )
            this.pgt.error("unexpected state");
        if( v1 )
            v2 = v2.v1;
        else
            v2 = v2.v2;       
    }
    v2 = context.calc(v2);
    switch(this.op)
    {
    case "&&" :
    case "||" :
    case "?" :
        return v2;
    case "+" :
        return v1 + v2;
    case "-" :
        return v1 - v2;
    case "!" : // XOR
        return v1 ? !v2 : v2 ;
    case "*" :
        return v1 * v2;
    case "/" :
        return v1 / v2;
    case ">" :
        return v1 > v2;
    case "<" :
        return v1 < v2;
    case ">=" :
        return v1 >= v2;
    case "<=" :
        return v1 <= v2;
    case "=" :
    case "==" :
        return v1 == v2;
    case "!=" :
        return v1 != v2;
    case "%" :
        return v1 % v2;
    }
    this.pgt.error("unexpected op: '" + this.op + "'");
    return 0;
};

InfixTerm_proto.pgtApplyTo = function(params)
{
    var v1 = applyTo(this.v1, params);
    var v2 = applyTo(this.v2, params);
    if( v1 === this.v1 && v2 === this.v2 )
        return this;
    return new InfixTerm(this.pgt, this.op, v1, v2);
};

InfixTerm_proto.pgtDependsOnCore = function(val)
{
    return this.pgt.exprDependsOn(this.v1, val) || this.pgt.exprDependsOn(this.v2, val);
};

InfixTerm_proto.pgtResolveCore = function(scope)
{
    this.pgt.exprResolve(this.v1, scope);
    this.pgt.exprResolve(this.v2, scope);
};

var UnitTerm = function(pgt, num, unit, dim)
{
    this.pgt = pgt;
    this.num = num;
    this.unit = unit;
    this.dim = dim;
};

var UnitTerm_proto = new Term(UnitTerm);
UnitTerm.prototype = UnitTerm_proto;

UnitTerm_proto.pgtCalcCore = function(context)
{
    return context.getUnit(this.unit, this.dim) * this.num;
};

UnitTerm_proto.toString = function()
{
    return this.num + this.unit;
};

var CustomTerm = function(pgt, func)
{
    this.pgt = pgt;
    this.func = func;
};

pgt.createCustomTerm = function(func)
{
    return new CustomTerm(this, func);
};

var CustomTerm_proto = new Term(pgt.CustomTerm);
CustomTerm.prototype = CustomTerm_proto;

CustomTerm_proto.pgtCalcCore = function(context)
{
    return this.func(context);
};

var ParamTerm = function(pgt, index)
{
    this.pgt = pgt;
    this.index = index;
};

var ParamTerm_proto = new Term(ParamTerm);
ParamTerm.prototype = ParamTerm_proto;

ParamTerm_proto.pgtCalcCore = function(context)
{
    this.pgt.error("unexpected state");
};

ParamTerm_proto.pgtApplyTo = function(params)
{
    return params[this.index];
};

var ApplyTerm = function(pgt, func, args)
{
    this.pgt = pgt;
    this.func = func;
    if( args.constructor == Array )
        this.args = args;
    else
    {
        this.args = [];
        while( args.constructor == InfixTerm && args.op == ',' )
        {
            this.args.splice(0,0,args.v2);
            args = args.v1;
        }
        this.args.splice(0,0,args);
    }
};

var ApplyTerm_proto = new Term(ApplyTerm);
ApplyTerm.prototype = ApplyTerm_proto;

ApplyTerm_proto.pgtCalcCore = function(context)
{
    var lambda = context.calc( this.func );
    var ctr = lambda.constructor;
    var this_args = this.args;
    if( ctr == LambdaTerm )
    {
        var func = applyTo( lambda.expr, this_args );
        return context.calc(func);
    }
    else if( ctr == Function )
    {
        var args = [];
        for( var i = 0 ; i < this_args.length ; i++ )
            args[i] = context.calc( this_args[i] );
        return lambda.apply(null, args);
    }
    else
        this.pgt.error("not a function!");
};

ApplyTerm_proto.pgtApplyTo = function(params)
{
    var clone = false;
    var args = [];
    for( var i = 0 ; i < this.args.length ; i++ )
    {
        args[i] = applyTo( this.args[i], params );
        if( args[i] !== this.args[i] )
            clone = true;
    }
    var func = applyTo( this.func, params );
    if( clone || func !== this.func )
        return new ApplyTerm( this.pgt, func, args );
    return this;
};

ApplyTerm_proto.pgtDependsOnCore = function(val)
{
    if( this.pgt.exprDependsOn( this.func, val ) )
        return true;
    for( var i = 0 ; i < this.args.length ; i++ )
        if( this.pgt.exprDependsOn( this.args[i], val ) )
            return true;
    return false;
};

ApplyTerm_proto.pgtResolveCore = function(scope)
{
    this.pgt.exprResolve( this.func, scope );
    for( var i = 0 ; i < this.args.length ; i++ )
        this.pgt.exprResolve( this.args[i], scope );
};

var LambdaTerm = function(pgt, expr)
{
    this.pgt = pgt;
    this.expr = expr;
};

var LambdaTerm_proto = new Term(LambdaTerm);
LambdaTerm.prototype = LambdaTerm_proto;

LambdaTerm_proto.pgtCalcCore = function(context)
{
    return this;
};

LambdaTerm_proto.pgtApplyTo = function(params)
{
    return this;
};

LambdaTerm_proto.pgtDependsOnCore = function(val)
{
    return this.pgt.exprDependsOn(this.expr, val);
};

LambdaTerm_proto.pgtResolveCore = function(scope)
{
    this.pgt.exprResolve(this.expr, scope);
};

var namePatt = /^\s*([a-zA-Z][\w\-]*)/;
var opPatt = /^\s*(\+|-|\*|\/|[>=<\!]=?|\&\&|\|\||\.|\?|\:|,)/;
var boPatt = /^\s*\(/;
var bcPatt = /^\s*\)/;
var escPatt = /^\s*\$([\w\-]*)/;
var wsPatt = /^\s+/;
var strPatt = /^\s*'([^']*)'|"([^"]*)"/;

var makePriorities = function()
{
    var pri = {};
    var p = 1;
    pri[","] = p;
    p++;
    pri["?"] = p;
    p++;
    pri[":"] = p;
    p++;
    pri["&&"] = p;
    pri["||"] = p;
    pri["!"] = p;
    p++;
    pri[">"] = p;
    pri[">="] = p;
    pri["<"] = p;
    pri["<="] = p;
    pri["="] = p;
    pri["=="] = p;
    pri["!="] = p;
    p++;
    pri["+"] = p;
    pri["-"] = p;
    p++;
    pri["*"] = p;
    pri["/"] = p;
    pri["%"] = p;
    p++;
    pri["@@"] = p; // apply
    p++;
    pri["."] = p;
    return pri;
};

var opPriority = makePriorities();

var reduceStack = function(pgt, stack, pri)
{
    var v = stack.pop();
    while( stack.length > 0 )
    {
        var op = stack.pop();
        if( op.match(/^P/) )
        {
            var opp = op.substr(1);
            v = new InfixTerm(pgt, opp, (opp=='!'?true:0), v);
            continue;
        }
        if( op == "(" )
        {
            if( pri !== 0 )
                stack.push("(");
            break;
        }
        var opPri = opPriority[op];
        if( opPri < pri )
        {
            stack.push(op);
            break;
        }
        var v2 = stack.pop();
        if( op == '.' )
            v = new AccessTerm(pgt, v2, v.name);
        else if( op == "@@" )
            v = new ApplyTerm(pgt, v2, v);
        else
            v = new InfixTerm(pgt, op, v2, v);
    }
    stack.push(v);                    
};

pgt.exprParse = function(text, esc, dim, lambda)
{
    var orig = text;
    var stack = [];
    var state = "val";
    var name, op;
    while( text.length > 0 )
    {
        var r;
        if( (r = text.match(this.patterns.cssLengthPart)) !== null )
        {
            var num = r[1] - 0;
            var unit = r[3];
            if( state != "val" )
            {
                this.error("could not parse: " + text );
                return null;
            }
            if( unit && unit != "" )
                num = new UnitTerm(this, num, unit, dim);
            stack.push(num);
            state = "op";
        }
        else if( (r = text.match(escPatt)) !== null )
        {
            name = r[1];
            if( state != "val" )
            {
                this.error("could not parse: " + text );
                return null;
            }
            var escval;
            if( name.match(/\d+/) )
            {
                if(!lambda)
                    this.error("unexpected parameter");
                escval = new ParamTerm(this, name-1); // zero-based index
            }
            else
            {
                escval = esc[name];
                if( escval === undef )
                    this.error("named value not found: " + name );
            }
            stack.push(escval);
            state = "op";
        }
        else if( (r = text.match(opPatt)) !== null )
        {
            op = r[1];
            if( state == "val" )
            {
                // prefix
                if( op != '-' && op != '!' )
                {
                    alert("could not parse: " + text );
                    return null;
                }
                stack.push("P" + op);
            }
            else
            {
                // infix
                reduceStack(this, stack, opPriority[op]);
                stack.push(op);
                state = "val";
            }
        }
        else if( (r = text.match(namePatt)) !== null )
        {
            name = r[1];
            if( state != "val" )
            {
                alert("could not parse: " + text );
                return null;
            }
            stack.push(new NameTerm(this, name));
            state = "op";
        }
        else if( (r = text.match(boPatt)) !== null )
        {
            if( state == "val" )
            {
                stack.push("(");
            }
            else
            {
                op = "@@";
                reduceStack(this, stack, opPriority[op]);
                stack.push(op);
                stack.push("(");
                state = "val";
            }
        }
        else if( (r = text.match(bcPatt)) !== null )
        {
            reduceStack(this, stack, 0);
        }
        else if( (r = text.match(strPatt)) !== null )
        {
            if( state != "val" )
            {
                alert("could not parse: " + text );
                return null;
            }
            var str = r[1]?r[1]:r[2];
            stack.push(str);
            state = "op";
        }
        else if( (r = text.match(wsPatt)) !== null )
        {
            // skip trailing space
        }
        else
        {
            alert("could not parse: " + text );
            return null;
        }
        text = text.substr(r[0].length);
    }
    reduceStack(this, stack, 0);
    if( stack.length != 1 )
    {
        this.error("could not parse: " + orig );
        return null;
    }
    var result = stack[0];
    if( lambda )
        result = new LambdaTerm(this, result);
    return result;
};

pgt.exprDependsOn = function(expr, val)
{
    var type = typeof expr;
    if( type == "object" && expr.pgtDependsOn !== undef )
        return expr.pgtDependsOn(val);
    else
        return false;    
};

pgt.exprResolve = function(expr, scope)
{
    var type = typeof expr;
    if( type == "object" && expr.pgtResolve !== undef )
        expr.pgtResolve(scope);
};

pgt.ExprScope = function(pgt)
{
    if( pgt )
        this.protectedInit(pgt);
};

pgt.createExprScope = function()
{
    return new this.ExprScope(this);
};

var ExprScope_proto = pgt.ExprScope.prototype;

ExprScope_proto.protectedInit = function(pgt)
{
    this.pgt = pgt;
    var nameTable = {};
    this.nameTable = nameTable;
    nameTable["page-width"] = new CustomTerm(pgt, function(cx) { return pgt.getViewportWidth(); });
    nameTable["page-height"] = new CustomTerm(pgt, function(cx) { return pgt.getViewportHeight(); });
};

ExprScope_proto.resolveAccess = function(obj, name)
{
    if( obj.pgtResolveAccess )
        return obj.pgtResolveAccess(name);
    this.pgt.error("no members");
};

var predefinedNames = {
    floor: Math.floor,
    ceil: Math.ceil,
    max: Math.max,
    min: Math.min,
    round: Math.round,
    sqrt: Math.sqrt
};

predefinedNames["true"] = true;
predefinedNames["false"] = false;

ExprScope_proto.protectedResolveName = function(name)
{
    var v = this.nameTable[name];
    if( v !== undef )
        return v;
    v = predefinedNames[name];
    if( v !== undef )
        return v;
    return undef;
};

ExprScope_proto.resolveName = function(name)
{
    var v = this.protectedResolveName(name);
    if( v === undef )
        this.pgt.error( "'" + name + "' undefined" );
    return v;
};

ExprScope_proto.define = function(name, value)
{
    var this_nameTable = this.nameTable;
    if( this_nameTable[name] !== undef )
        this.pgt.error("'" + name + "' already defined");
    else
        this_nameTable[name] = value;
};

ExprScope_proto.resolve = function()
{
    var this_nameTable = this.nameTable;
    for( var name in this_nameTable )
        this.pgt.exprResolve(this_nameTable[name], this);
};

var ExprContext = function(pgt)
{
    if( pgt )
        this.protectedInit(pgt);
};

pgt.createExprContext = function()
{
    return new ExprContext(this);
};

var ExprContext_proto = ExprContext.prototype;

ExprContext_proto.protectedInit = function(pgt)
{
    this.pgt = pgt;
};

ExprContext_proto.getViewportWidth = function()
{
    return this.pgt.getViewportWidth();
};

ExprContext_proto.getViewportHeight = function()
{
    return this.pgt.getViewportHeight();
};

ExprContext_proto.getUnit = function(name, dim)
{
    switch( name )
    {
        case "px" :
            return 1;
        case "rpx" :
            return this.pgt.units.rpx;
        case "em" :
        case "rem" :
            return this.pgt.units.em;
        case "ex" :
            return this.pgt.units.ex;
        case "pt" :
            return this.pgt.units.pt;
        case "in" :
            return this.pgt.units.pt*72;
        case "cm" :
            return this.pgt.units.pt*28.346456692913385;
        case "mm" :
            return this.pgt.units.pt*2.8346456692913385;
        case "%" :
            if( dim !== null && dim !== undef && dim.match(/^(height|top|bottom)$/) )
                return this.getViewportHeight()/100.0;
            return this.getViewportWidth()/100.0;
        default:
            this.pgt.error("unknown unit: " + name);
    }
    return 1;
};

ExprContext_proto.calc = function(expr)
{
    if( expr !== null && typeof expr == "object" && expr.pgtCalc !== undef )
        return expr.pgtCalc(this);
    else
        return expr;
};

})(pgt);