// Page Template Library
// Fitting text in the region(s)

// begin private scope
(function(pgt, undef) {

/*
AdobePatentID="B1278"
*/

if( !pgt )
    alert("pgt-util.js is required");
pgt.provides("pgt-fit");

var TextFitter = function(pgt, template, props)
{
    this.pgt = pgt;
    this.template = template;
    this.props = props;
    this.locked = true;
};

pgt.createTextFitter = function(template, props)
{
    return new TextFitter(this, template, props);
};

var TextFitter_proto = TextFitter.prototype;

TextFitter_proto.reset = function(dryRun)
{
    this.dryRun = dryRun;
    if( dryRun )
    {
        this.fragmentData = [];
        this.checkpoints = [];
    }
    this.currGeomPos = 0;
    this.checkpointIndex = 1;
    this.stat = " ";
};

TextFitter_proto.makeGeomPos = function( y )
{
    var floats = this.floats;
    var pos = this.currGeomPos;
    var width = this.width;
    if( floats )
    {
        var len = floats.length;
        if( len > 0 )
        {
            var i = 0;
            var lastY = 0;
            while( true )
            {
                var h = y - lastY;
                var f;
                var w = this.width;
                if( i >= len || (f = floats[i++]).y >= y )
                    return pos + h*w;
                pos += (f.y - lastY)*w;
                w -= f.width;
                if( f.x == 0 && i < len )
                {
                    // check if there is right float
                    var f1 = floats[i];
                    if( f1.y == f.y )
                    {
                        i++;
                        w -= f1.width;
                    }
                }
                lastY = f.y + f.height;
                if( y <= lastY )
                    return pos + (y - f.y)*w;
                pos += f.height*w;
            }
        }
    }
    return pos + y*width;
};

TextFitter_proto.beginFragment = function( container, width, height, floats, modeStyle )
{
    if( this.dryRun )
    {
        var data = { container:container, width:width, height:height, floats:floats, modeStyle:modeStyle };
        this.fragmentData.push(data);
    }
    this.width = width;
    this.height = height;
    this.floats = floats;
};

TextFitter_proto.endFragment = function(width, height)
{
    this.currGeomPos = this.makeGeomPos(height);
};

TextFitter_proto.addCheckpoint = function( fragmentY, charPos )
{
    var geomPos = this.makeGeomPos( fragmentY );
    var checkpoint = { geomPos:geomPos, charPos:charPos };
    this.checkpoints.push(checkpoint);
};

TextFitter_proto.prepare = function(context)
{
    this.lineHeight = context.calc(this.props["flow-line-height"]);
    if( !this.lineHeight )
        this.lineHeight = 24*this.pgt.units.rpx;
    var target = context.calc(this.props["fitting-target"]);
    var arr = (target + "").split(/\s+/);
    if( arr.length == 2 && arr[1] == "pages" )
    {
        if( arr[0] == "whole" )
            this.wholePages = true;
        else
            this.targetPages = arr[0] - 0;
    }
};

TextFitter_proto.getTargetPages = function()
{
    return this.targetPages;
};

TextFitter_proto.targetReached = function()
{
    // shoot for a fairly short line at the end
    this.targetGeomPos = this.currGeomPos - 0.9*this.lineHeight*this.width;
};

TextFitter_proto.layoutDone = function()
{
    if( this.dryRun )
    {
        var checkpoints = this.checkpoints;
        if( checkpoints.length > 0 )
        {
            var last = checkpoints[checkpoints.length-1];
            this.lastCharPos = last.charPos;
            this.lastGeomPos = last.geomPos;
            this.pgt.status( "stretchFactor", (this.targetGeomPos - this.lastGeomPos)/this.lastGeomPos );
        }
    }
};

var randomRound = function(x)
{
    return Math.floor(x+Math.random());
};

// returns extra area per character that needs to be consumed
TextFitter_proto.getTargetAdjustments = function( fragmentY, charPos, size, wordCount )
{
    var cp0;
    var cp1 = this.checkpoints[this.checkpointIndex];
    while( cp1.charPos < charPos )
    {
        cp0 = cp1;
        this.checkpointIndex++;
        cp1 = this.checkpoints[this.checkpointIndex];
    }
    if( !cp0 )
        cp0 = this.checkpoints[this.checkpointIndex-1];
    var dryRunGeomPos;
    if( cp1.charPos == charPos )
        dryRunGeomPos = cp1.geomPos;
    else
        dryRunGeomPos = cp0.geomPos + (cp1.geomPos - cp0.geomPos)*(charPos - cp0.charPos)/(cp1.charPos - cp0.charPos);
        
    var actualGeomPos = this.makeGeomPos( fragmentY );;
    var remainsChars = this.lastCharPos - charPos;
    var remainsGeom = this.targetGeomPos - actualGeomPos;
    var remainsDryRunGeom = this.lastGeomPos - dryRunGeomPos;
    var excessGeom = remainsGeom - remainsDryRunGeom;
    
    // per char
    var atEnd = remainsChars < 1.5*size;
    var adjustmentGeom = excessGeom/remainsChars;
    var baseGeom = remainsDryRunGeom/remainsChars;

    // try to distribute adjustment across the next chunk of content 
    
    var lineHeight = this.lineHeight;        
    var adjLength = adjustmentGeom/lineHeight;
    var baseLength = baseGeom/lineHeight;
    
    var adj = {};
    
    var round = Math.round;
            
    var rpx = this.pgt.units.rpx;
    
    this.stat += "[S:" + size + " A:" + Math.round(adjLength);
    if( wordCount > 0 )
    {
        var charsPerWord = Math.ceil(size / (wordCount-1));
        var wordSpacing = charsPerWord*adjLength;
        if( wordSpacing < 0 )
            wordSpacing = Math.max(-lineHeight/24,Math.ceil(wordSpacing));
        else
            wordSpacing = Math.min(lineHeight/8,round(wordSpacing));
        if( wordSpacing != 0 )
        {
            adj.wordSpacing = wordSpacing + "px";
            this.stat += " W:" + wordSpacing;
            adjLength -= wordSpacing/charsPerWord;
        }
    }

    if( Math.abs(adjLength) > lineHeight/48 )
    {
        // have to adjust font size
        var ratio = 1 + adjLength/baseLength;
        ratio = Math.max(0.92, Math.min(1.09, ratio));
        ratio = round(ratio*100)/100;
        adj.fontSize = ratio + "em";
        this.stat += " F:" + ratio;
    }
    else
    {
        var letterSpacing;
        if( adjLength < 0 )
            letterSpacing = Math.ceil(adjLength);
        else
            letterSpacing = round(adjLength);
        if( letterSpacing != 0 )
        {
            adj.letterSpacing = letterSpacing + "px";
            this.stat += " L:" + letterSpacing;
        }
    }
    this.stat += "] ";
    return adj;
};

TextFitter_proto.isDryRun = function()
{
    return this.dryRun;
};

TextFitter_proto.fit = function(flow)
{
    var fd = this.fragmentData;
    var elemFunc;
    for( var i = 0 ; i < fd.length ; i++ )
    {
        var data = fd[i];
        var container = data.container;
        while( container.firstChild )
            container.removeChild(container.firstChild);
        var modeStyle = data.modeStyle;
        if( modeStyle )
            elemFunc = modeStyle.getApplyViewStylesFunc();
        else
            elemFunc = this.pgt.applyViewStyles;
        this.beginFragment( container, data.width, data.height, data.floats, modeStyle );
        flow.flowInto(container, null, data.floats, elemFunc, this);
        this.endFragment(data.width, data.height);
    }  
    this.pgt.status( "finalGeomPos", this.currGeomPos );
    this.pgt.status( "targetGeomPos", this.targetGeomPos );
    if( this.stat )
        this.pgt.status( "fitterStat", this.stat );
};

})(pgt);

