// Page Template Library
// flow HTML in a set of boxes

// begin private scope
(function(pgt, undef) {

if( !pgt )
    alert("pgt-util.js is required");
pgt.provides("pgt-flow");

var FlowItem = function( pgt, sourceElement, offset )
{
    this.pgt = pgt;
    this.element = sourceElement;
    this.sourceDone = false;
    this.offset = offset;
    this.options = {};
    this.filter = function(e) { return true; };
    this.getPropertyFunc = function(e) { return null; };
};

pgt.createFlowItem = function(sourceElement, offset)
{
    return new FlowItem(this, sourceElement, offset);
};

var FlowItem_proto = FlowItem.prototype;

FlowItem_proto.setOption = function( name, value )
{
    if( this.flow )
    {
        this.pgt.error("Item already added");
        return;
    }
    this.options[name] = value;
};

FlowItem_proto.setFilter = function( filter )
{
    if( this.flow )
    {
        this.pgt.error("Item already added");
        return;
    }
    this.filter = filter;
};

FlowItem_proto.setGetPropertyFunc = function( func )
{
    if( this.flow )
    {
        this.pgt.error("Item already added");
        return;
    }
    this.getPropertyFunc = func;
};

FlowItem_proto.getSourceElement = function()
{
    return this.element;
};

FlowItem_proto.privateForceJustify = function()
{
    if( !this.lastBrokenContainer )
        return;
        
    var doc = this.pgt.document;
    
    // add a spacer to force justification in the last line
    var spacer = doc.createElement("span");
    if( this.pgt.isWebKit() )
    {
        // WebKit does not justify last line correctly with inline block...
        spacer.style.visibility = "hidden";
        spacer.appendChild(doc.createTextNode("XXXXXXXXXXXXXXX"));
    }
    else
    {
        spacer.style.display = "inline-block";
        spacer.style.width = "95%";
        spacer.style.height = "1px";
        spacer.style.lineHeight = "1px";
    }
    this.lastBrokenText.data += " ";
    this.lastBrokenContainer.appendChild(spacer);
};

FlowItem_proto.addView = function(node, view, wholesale, proxyTarget)
{
    if( this.tracker )
        this.tracker.addView(node, view, wholesale, proxyTarget);
    this.pgt.addView(node, view, wholesale, proxyTarget);
};

FlowItem_proto.privateQueueExceptionItem = function()
{
    var newItem = this.pgt.createFlowItem(this.element, this.offset);
    newItem.isException = true;
    newItem.options = this.options;
    newItem.filter = this.filter;
    newItem.getPropertyFunc = this.getPropertyFunc;
    newItem.flow = this.flow;
    newItem.tracker = this.tracker;
    newItem.flowEntry = this.flowEntry.createExceptionEntry();
    var items = this.flow.items;
    var i = 0;
    while( i < items.length && items[i].isException )
        i++;
    items.splice(i, 0, newItem);
};

var Flow = function(pgt)
{
    this.pgt = pgt;
    this.items = [];
    this.itemFilter = function(e) { return true; };
    this.charPos = 0;
};

pgt.createFlow = function()
{
    return new Flow(this);
};

var Flow_proto = Flow.prototype;

Flow_proto.addItem = function( item )
{
    item.flow = this;
    this.items.push(item);
};

Flow_proto.getItemCount = function()
{
    return this.items.length;
};

Flow_proto.getItem = function(index)
{
    return this.items[index];
};

Flow_proto.setItemFilter = function( filter )
{
    this.itemFilter = filter;
};

Flow_proto.privateNextItemIndex = function()
{
    var items = this.items;
    var index = -1;
    var exclusiveIndex = -1;
    var exclusivePriority = undef;
    for( var i = 0 ; i < items.length ; i++ )
    {
        var item = items[i];
        if( item.processed || !this.itemFilter(item) )
            continue;
        var opt = items[i].options;
        if( opt.exclusive )
        {
            if( exclusivePriority === undef || (opt.priority !== undef && exclusivePriority <= opt.priority) )
            {
                exclusiveIndex = i;
                exclusivePriority = opt.priority;
            }
        }
        else
        {
            if( index < 0 )
                index = i;
        }
    }
    if( exclusiveIndex >= 0 )
        return exclusiveIndex;
    return index;
};

Flow_proto.removeItem = function( index )
{
    var item = this.items[index];
    this.items.splice(index, 1);
};

Flow_proto.getFlowPosition = function()
{
    if( this.items.length === 0 )
        return null;
    var item = this.items[0];
    var fe = item.flowEntry;
    if( !fe )
        return {node:item.element, offset:0};
    while( fe.child )
        fe = fe.child;
    return {node:fe.xmlnode, offset:fe.offset};
};

var addFloat = function(elem, width, height, side)
{
    var fc = elem.ownerDocument.createElement("div");
    fc.style.width = width + "px";
    fc.style.height = height + "px";
    fc.style.cssFloat = side;
    fc.style.clear = side;
    //fc.style.outline = "1px solid red";
    elem.appendChild(fc);
};

var addFloats = function(pgt, elem, floats, side)
{
    if( !floats )
        return;
    var y = 0;
    var i = 0;
    while( i < floats.length )
    {
        var f = floats[i++];
        var f1 = (i < floats.length && floats[i].y == f.y ? floats[i++] : null);
        if( f.y > y )
        {
            addFloat(elem, 0, f.y - y, "left" );
            addFloat(elem, 0, f.y - y, "right" );
        }
        if( f.x === 0 )
        {
            addFloat(elem, f.width, f.height, "left");
            if( f1 === null )
                addFloat(elem, 0, f.height, "right");
            else
                addFloat(elem, f1.width, f1.height, "right");
        }
        else
        {
            if( f1 !== null )
                pgt.error("unexprected state");
            addFloat(elem, 0, f.height, "left");                    
            addFloat(elem, f.width, f.height, "right");
        }
        y = f.y + f.height;
    }
};

Flow_proto.flowInto = function( targetElement, rect, floats, styleFunc, fitter, forceFit )
{
    if( !rect )
        rect = this.pgt.getBoundingClientRect(targetElement);
    addFloats( this.pgt, targetElement, floats );
    var container = this.pgt.document.createElement("div");
    var item, res;
    var processedItems = [];
    targetElement.appendChild(container);
    while(this.items.length > 0)
    {
        var itemIndex = this.privateNextItemIndex();
        if( itemIndex < 0 )
            break;
        item = this.items[itemIndex];
        if( !item.flowEntry )
            item.flowEntry = new FlowEntry(this.pgt, item, null, item.element, 0);
        item.lastBrokenContainer = null;
        item.fitter = fitter;
        item.styleFunc = styleFunc;
        item.charPos = this.charPos;
        item.tracker = this.tracker;
        item.forceFit = forceFit;
        res = item.flowEntry.flowNode(container, rect, 0);
        this.charPos = item.charPos;
        forceFit = item.forceFit;
        if( item.options.exclusive && !item.options["static"] )
            this.removeItem(itemIndex);
        if( item.options.exclusive || res != 1 )
            break;
        if( item.options["static"] )
        {
            processedItems.push(item);
            item.processed = true;
        }
        else
            this.removeItem(itemIndex);
    }
    for( var i = 0 ; i < processedItems.length ; i++ )
        processedItems[i].processed = false;
    var crect = this.pgt.getBoundingClientRect(container);
    var h = crect.bottom - rect.top;
    if( item )
        item.privateForceJustify();
    return h;
};

Flow_proto.beginRegion = function(width, height, columnWidth)
{
};

Flow_proto.endRegion = function()
{
};

Flow_proto.beginPage = function(tracker)
{
    // setup items that are not filtered out as started
    this.tracker = tracker;
    var items = this.items;
    for( var i = 0 ; i < items.length ; i++ )
    {
        var item = items[i];
        if( !this.itemFilter(item) )
            continue;
        if( item.activePageCount === undef )
            item.activePageCount = 0;
        else
            item.activePageCount++;
    }
};

Flow_proto.endPage = function()
{
    // kill some started items based on linger
    var items = this.items;
    var i = 0;
    while( i < items.length )
    {
        var item = items[i];
        if( this.itemFilter(item) )
        {
            if( item.options.linger !== undef )
            {
                if( item.activePageCount >= item.options.linger )
                {
                    items.splice(i, 1);
                    continue;
                }
            }
        }
        i++;
    }
    this.tracker = null;
};

var breakOffElementBottom = function(e)
{
    var style = e.style;
    if( style.marginBottom || style.paddingBottom || style.borderBottom )
    {
        e.bottomStyle = {marginBottom: style.marginBottom, paddingBottom:style.paddingBottom,
             borderBottom:style.borderBottom };
    }
    style.marginBottom = "0px";
    style.paddingBottom = "0px";
    style.borderBottom = "none";
};

var restoreElementBottom = function(e)
{
    var style = e.style;
    var bs = e.bottomStyle;
    if( bs )
    {
        style.marginBottom = bs.marginBottom;
        style.paddingBottom = bs.paddingBottom;
        style.borderBottom = bs.borderBottom;
    }
    else
    {
        style.marginBottom = "";
        style.paddingBottom = "";
        style.borderBottom = "";
    }
};

var breakOffElementTop = function(e)
{
    e.style.marginTop = "0px";
    e.style.paddingTop = "0px";
    e.style.borderTop = "none";
    e.style.textIndent = "0px";
};

var nextWordBreak = function(text, index)
{
    var len = text.length;
    // skip spaces
    while( index < len && text.charCodeAt(index) <= 0x20 )
        index++;
    // skip non-spaces
    while( index < len && text.charCodeAt(index) > 0x20 )
        index++;
    return index;
};

var prevWordBreak = function(text, index)
{
    if( index <= 0 )
        return 0;
    index--;
    // skip non-spaces
    while( text.charCodeAt(index) > 0x20 )
    {
        if( index === 0 )
            return 0;
        index--;
    }
    // skip spaces
    while( text.charCodeAt(index) <= 0x20 )
    {
        if( index === 0 )
            return 0;
        index--;
    }
    return index+1;
};

var getWordCount = function(text)
{
    var count = 0;
    var index = 0;
    var len = text.length;
    while( index < len )
    {
        index = nextWordBreak(text, index);
        count++;
    }
    return count;
};

var FlowEntry = function(pgt, item, parent, xmlnode, offset)
{
    this.pgt = pgt;
    this.owner = item;
    this.parent = parent;
    this.xmlnode = xmlnode;
    this.offset = offset;
    if( parent )
        parent.child = this;
};

var FlowEntry_proto = FlowEntry.prototype;

FlowEntry_proto.flowNode = function(container, rect, depth)
{
    var res;
    switch( this.xmlnode.nodeType )
    {
    case 1:
        return this.flowElement(container, rect, depth);
    case 3:
    case 4:
    case 5:
        return this.flowText(container, rect);
    }
    this.parent.child = null;
    this.parent = null;
    return 1;
};

FlowEntry_proto.flowText = function(container, rect)
{
    if( !this.parent || this.parent.child !== this )
    {
        this.pgt.error("unexpected state");
        return 0;
    }
    
    var owner = this.owner;
    var node = this.xmlnode;
    var text = node.data;
    
    var str = text.substr(this.offset);        
    var res = this.flowTextInner(container, rect, container, str);
    if( res == 1 )
    {
        this.parent.child = null;
        this.parent = null;
    }
    return res;
};

FlowEntry_proto.flowTextInner = function(container, rect, appendPoint, str)
{    
    var owner = this.owner;
    var doc = this.pgt.document;
    var view = doc.createTextNode(str);
    var r = this.pgt.getBoundingClientRect(container);
    var fragmentY;
            
    if( owner.fitter && !owner.fitter.isDryRun() )
    {
        fragmentY = 0;
        if( r && r.bottom > rect.top )
            fragmentY = r.bottom - rect.top;
            
        var p = appendPoint;
        appendPoint = doc.createElement("span");
        p.appendChild(appendPoint);
        var style = appendPoint.style;
        
        var adj = owner.fitter.getTargetAdjustments( fragmentY, owner.charPos, str.length, getWordCount(str) );
                
        style.letterSpacing = adj.letterSpacing;
        style.wordSpacing = adj.wordSpacing;
        style.fontSize = adj.fontSize;        
    } 
    
    var bottom0 = r.bottom;
    appendPoint.appendChild(view);
    r = this.pgt.getBoundingClientRect(container);
    if( r.bottom <= rect.bottom )
    {
        // whole thing fits
        owner.charPos += str.length;
        if( owner.fitter && owner.fitter.isDryRun() )
        {
            fragmentY = r.bottom - rect.top;
            owner.fitter.addCheckpoint( fragmentY, owner.charPos );
        }
        return 1;
    }
    
    // interpolate
    var index = Math.floor( (rect.bottom - bottom0)*str.length/(r.bottom - bottom0) );
    index = prevWordBreak( str, index );
    
    var direction = 0;
    var prevIndex;
    
    while(true)
    {
        view.data = str.substr(0,index);
        r = this.pgt.getBoundingClientRect(container);
        if( direction === 0 )
        {
            if( r.bottom > rect.bottom )
                direction = -1; // backward: chop off some text until it fits 
            else
                direction = 1; // forward: add more text until it overflows
        }
        else
        {
            if( direction > 0 )
            {
                if( r.bottom > rect.bottom )
                {
                    // going forward: now it does not fit anymore!
                    if( prevIndex == 0 )
                    {
                        appendPoint.removeChild(view);
                        return 0; // nothing                
                    }
                    view.data = str.slice(0, prevIndex);
                    index = prevIndex;
                    break;
                }
            }
            else
            {
                if( r.bottom <= rect.bottom )
                {
                    // going backward: now it fits!
                    break;
                }
            }
        }            
        if( index == str.length )
        {
            // hmm, why didn't it fit before?
            this.pgt.error("Error4");
        }
        prevIndex = index;
        if( direction > 0 )
        {
            index = nextWordBreak(str, index);
        }
        else
        {
            index = prevWordBreak(str, index);
            if( index === 0 )
            {
                appendPoint.removeChild(view);
                return 0; // nothing fits              
            }
        }
    }

    this.offset += index;
    owner.charPos += index;
    if( owner.fitter && owner.fitter.isDryRun() )
    {
        var fragmentY = r.bottom - rect.top;
        owner.fitter.addCheckpoint( fragmentY, owner.charPos );
    }
        
    owner.lastBrokenText = view;
    owner.lastBrokenContainer = container;
    
    return this;
};

var isNonBreakable = function(node, owner)
{
    var breakInside = owner.getPropertyFunc(node, "break-inside");
    if( breakInside == "avoid" || breakInside == "avoid-column")
        return true;
    return false;
};

var nonEmptyNode = function(node)
{
    return node.nodeType != 3 || node.data.match(/^\s*$/) == null;
};

var applyToChildren = function(func, view, node, proxyTarget)
{
    view = view.firstChild;
    node = node.firstChild;
    while( view )
    {
        if( view.nodeType == 1 )
        {
            func(view, node, proxyTarget);
            applyToChildren(func, view, node, proxyTarget);
        }
        view = view.nextSibling;
        node = node.nextSibling;
    }
};

FlowEntry_proto.flowElement = function(container, rect, depth)
{
    var node = this.xmlnode;
    var owner = this.owner;
    var r;
    if( !owner.filter(node, depth) )
    {
        if( this.parent )
        {
            this.parent.child = null;
            this.parent = null;
        }
        return 1; // consume
    }
    depth++;
    
    var proxyTarget = node.pgtProxyTarget;
    var wholesale = flexible || isNonBreakable(node, owner);
    if( proxyTarget )
    {
    	wholesale = true;
    	if( proxyTarget.ownerDocument != container.ownerDocument )
    	{
    		var origProxyTarget = proxyTarget;
    		proxyTarget = node.pgtProxyTargetImported;
    		if( !proxyTarget || proxyTarget.ownerDocument != container.ownerDocument )
    		{
    			proxyTarget = this.pgt.createView(origProxyTarget, true);
     			proxyTarget.pgtProxy = node;
    			node.pgtProxyTargetImported = proxyTarget;
    		}
    		else
    			proxyTarget.setAttribute("style", origProxyTarget.getAttribute("style"));
    	}
    }
    var flexible = owner.getPropertyFunc(node, "region-placement") == "flexible";
    var view = this.pgt.createView(node, wholesale);
    if( owner.styleFunc )
    {
        owner.styleFunc(view, node, proxyTarget);
        if( wholesale )
            applyToChildren(owner.styleFunc, view, node, proxyTarget);
    }
    container.appendChild(view);
    r = this.pgt.getBoundingClientRect(view); // should we always look for container?? - does not work for floats
    if( r.bottom > rect.bottom && !owner.forceFit )
    {
        container.removeChild(view);
        if( depth > 0 && flexible )
        {
            owner.privateQueueExceptionItem(); // schedule for later
            if( this.parent )
            {
                this.parent.child = null;
                this.parent = null;
            }
            return 1; // consume
        }
        return 0; // nothing
    }
    if( wholesale )
    {
        //if( proxyTarget )
        //    view.style.background = "yellow";
        owner.addView(node, view, wholesale, proxyTarget);
    }
    else
    {
        // delay owner.addView call until we have first child added
        var committed = false;
        var xmlchild;
        var origChild = this.child;
        if( origChild )
            xmlchild = origChild.xmlnode;
        else
            xmlchild = node.firstChild;
        for( ; xmlchild; xmlchild = xmlchild.nextSibling )
        {
            if( !this.child )
                new FlowEntry(this.pgt, owner, this, xmlchild, 0);
            else
                breakOffElementTop(view);
            var res = this.child.flowNode(view, rect, depth);
            if( res === 0 )
            {
                if( !committed )
                {
                    // roll back
                    container.removeChild(view);
                    this.child = origChild;
                    return 0;
                }
                owner.forceFit = false;
                return this.child;
            }
            if( !committed && nonEmptyNode(xmlchild) )
            {
                committed = true;
                owner.forceFit = false;
                owner.addView(node, view, wholesale);
            }
            if( res != 1 )
            {
                if( !committed )
                    owner.addView(node, view, wholesale);
                breakOffElementBottom(view);
                return res;
            }
            if( this.child )
            {
                this.pgt.error("unexpected state");
                return 0;
            }
            if( this.isException )
                break;
        }
        if( !committed )
            owner.addView(node, view, wholesale);
    }
    if( this.parent )
    {
        this.parent.child = null;
        this.parent = null;
    }
    return 1; // the whole thing
};

FlowEntry_proto.createExceptionEntry = function(parent)
{
    var entry = new FlowEntry(this.pgt, this.owner, parent, this.xmlnode, this.offset);
    if( this.child )
    {
        entry.child = this.child.createExceptionEntry(entry);
        entry.isException = true;
    }
    return entry;
};

})(pgt);
