// Page Template Library
// hooks to jQuery

// begin private scope
(function(jQuery, pgt, undef) {

if( !pgt )
    alert("pgt-util.js is required");
pgt.provides("pgt-jquery");

if( !jQuery )
{
    pgt.error("jQuery is required");
    return;
}

var hookEvents = function(pgt, page)
{
    var eventListener = function(evt)
    {
        var view = evt.target;
        var master = pgt.getMaster(view);
        if( master )
            pgt.trigger(master, evt.type, evt);
    };
    page.addEventListener( "click", eventListener, false );
    page.addEventListener( "mousedown", eventListener, false );
    page.addEventListener( "mouseup", eventListener, false );
};

var privateTrigger = function(pgt, target, name, param)
{
    if( target === document )
    {
        switch(name)
        {
        case "master-created.pgt" :
            param.className = "pgt-master";
            break;
        case "page-set-current.pgt" :
            var pid = pgt.privateInstanceData;
            if( pid.currentPage )
                pid.currentPage.className = "pgt-page";
            pid.currentPage = param;
            pid.currentPage.className = "pgt-page pgt-current-page";
            break;
        case "page-created.pgt" :
            param.className = "pgt-page";
            hookEvents(pgt, param);
            break;
        }
    }
};

pgt.trigger = function(target, name, param )
{
    privateTrigger(this, target, name, param);
    jQuery(target).trigger(name, [param]);
};

pgt.privateCreateViewOrig = pgt.createView;

pgt.createView = function(elem, recursive)
{
    var view = this.privateCreateViewOrig(elem, recursive);
    if( view.className == "pgt-master" )
        view.className = "";
    return view;
}; 

pgt.addView = function(master, view, recursive)
{
    var views = master.pgtViews;
    if( views )
        views.push(view);
    else
        master.pgtViews = [view];
    view.pgtMaster = master;
    this.trigger(master, "view-added.pgt", view); 
};

pgt.clearViews = function(elem)
{
    delete elem.pgtViews;
    this.trigger(elem, "view-cleared.pgt"); 
};

pgt.getViews = function(master)
{
    return master.pgtViews;
};

pgt.getMaster = function(view)
{
    return view.pgtMaster;
};

var plugin = {
    views: function(self) {
        var res = [];
        self.each( function() {
            var views = this.pgtViews;
            if( views )
                Array.prototype.push.apply(res,views);
        });
        return jQuery(res);
    },
    addViews: function(self) {
        var res = [];
        self.each( function() {
            var views = this.pgtViews;
            if( views )
                Array.prototype.push.apply(res,views);
            res.push(this);
        });
        return jQuery(res);
    }
};

jQuery.fn.pgt = function(what) {
    if( !what )
        what = "addViews";
    return plugin[what](this);
};

})(jQuery, pgt);