// begin private scope
(function(pgt, undef) {

if( !pgt )
    alert("pgt-util.js is required");
pgt.provides("pgt-page");

var Page = function(pgt, stacking)
{
    var doc = pgt.document;
    var page = doc.createElement("div");
    this.page = page;
    var ps = page.style;
    ps.width = pgt.getViewportWidth() + "px";
    ps.height = pgt.getViewportHeight() + "px";
    ps.backgroundColor = "white";
    ps.fontSize = pgt.units.em + "px";
    
    if( pgt.units.rpx != 1 )
    {
	    var tsf = "scale(" + 1.0/pgt.units.rpx + ")";
	    ps.WebkitTransform = tsf;
	    ps.WebkitTransformOrigin = "0 0";
	    ps.MozTransform = tsf;
	    ps.MozTransformOrigin = "0 0";
	    ps.OTransform = tsf;
	    ps.OTransformOrigin = "0 0";
    }
    
    if( pgt.isPrinting() )
        stacking = "print";
    switch( stacking )
    {
    case "horizontal" :
    	// TODO: correct coordinate
        ps.left = (0 * pgt.getViewportWidth()/pgt.units.rpx) + "px";
        ps.position = "absolute";
        break;
    case "vertical" :
        ps.position = "relative";
        break;
    case "print" :
        var margin = pgt.getPrintMargin();
        ps.position = "relative";
        ps.marginTop = margin + "px";
        ps.marginLeft = margin + "px";
        ps.pageBreakInside = "avoid";
        ps.pageBreakAfter = "always";
        break;
    default:
        ps.position = "absolute";
        ps.visibility = "hidden";
        break;
    }    
};

pgt.PagedView = function(pgt)
{
    if( pgt )
        this.protectedInit(pgt);
};

pgt.createPagedView = function()
{
    return new this.PagedView(this);
};

var PagedView_proto = pgt.PagedView.prototype;

PagedView_proto.protectedInit = function(pgt)
{
    this.pgt = pgt;
    this.chunks = [];
    this.currentChunkIndex = 0;
    this.currentPageIndex = 0;
    this.tick = 1;
};

PagedView_proto.navigationHandler = function(url)
{
	// handle local urls
    if( url.match(pgt.patterns.idfragment) )
        return url;
    return null;
};

PagedView_proto.setPageProducer = function(producer)
{
    this.pageProducer = producer;
};

PagedView_proto.setNavigationHandler = function(navigationHandler)
{
	this.navigationHandler = navigationHandler;
};

PagedView_proto.setPageTransition = function(transition, stacking)
{
    this.pageTransition = transition;
    if( stacking )
    	this.stacking = stacking;
};

PagedView_proto.processHyperlinkClick = function(evt)
{
    var target = evt.target;
    var localName = target.localName;
    if( localName != "a" )
        return;
    var href = target.getAttribute("href");
    if( !href )
        href = target.getAttributeNS( "http://www.w3.org/1999/xlink", "href" ); // SVG
    if( href )
    {
        var bookmark = this.navigationHandler(href);
        if( bookmark )
        {
        	var loc = {bookmark:href};
        	this.setCurrentLocation(loc);
        }
    }
    evt.preventDefault();
};

PagedView_proto.getChunk = function(chunkIndex)
{
	var chunk = this.chunks[chunkIndex];
	if( !chunk )
	{
		chunk = {pages:[]};
		this.chunks[chunkIndex] = chunk;
	}
	return chunk;
};

PagedView_proto.getCurrentChunk = function()
{
	return this.getChunk(this.currentChunkIndex);
};

PagedView_proto.getPage = function(chunkIndex, pageIndex)
{
	var chunk = this.chunks[chunkIndex];
	if( chunk )
		return chunk.pages[pageIndex];
	return null;
};

PagedView_proto.getCurrentPage = function()
{
	return this.getPage(this.currentChunkIndex, this.currentPageIndex);
};

PagedView_proto.privateBuildNextPageInChunk = function(chunkIndex)
{
	var chunk = this.getChunk(chunkIndex);
	if( chunk.done )
		return null;
    
    if( !this.stacking )
        this.stacking = "single";
    
    var pgt = this.pgt;
    var pageObj = new Page(pgt, this.stacking);
    var page = pageObj.page;
    
    // TODO: order pages across chunks
    pgt.body.insertBefore(page, this.overlayLayer);
    
    if( !this.pageProducer.buildNextPage(chunkIndex, chunk.pages.length, pageObj) )
    {
    	pgt.body.removeChild(page);
    	chunk.done = true;
    	return null;
    }

    if (pgt.needEvents)
	{
	    if( !this.hyperlinkClickFilter)
	    {
	    	var self = this;
	        this.hyperlinkClickFilter = function(evt) {self.processHyperlinkClick(evt);};
	    }

	    page.addEventListener( "click", this.hyperlinkClickFilter, false );	
	}

    chunk.pages.push(pageObj);
    this.pgt.trigger(this.pgt.rootEventTarget, "page-created.pgt", page);    
    return page;
};

PagedView_proto.reset = function()
{
	var prev = this.getCurrentPage();
	this.setProxyVisibility(prev, false);
    this.currentPageIndex = 0;
    this.currentChunkIndex = 0;
    this.tick++;
    if( !this.chunks )
        this.chunks = [];
    else
    {
    	while( this.chunks.length > 0 )
    	{
    		this.clearChunkPages(this.chunks.pop(), 0);
    	}
    }
    this.pageProducer.reset();
};

PagedView_proto.clearChunkPages = function(chunk, pageCount)
{
	if( !chunk )
		return;
	var pages = chunk.pages;
    while( pages.length > pageCount )
    {
        var pageObj = pages.pop();
        var page = pageObj.page;
        page.parentNode.removeChild(page);
    }
};

PagedView_proto.buildOverlayLayer = function()
{
    var pgt = this.pgt;
    var doc = pgt.document;
    this.overlayLayer = doc.createElement("div");
    pgt.body.appendChild(this.overlayLayer);
};

PagedView_proto.getCurrentLocation = function()
{
	var loc = {
		chunkIndex:this.currentChunkIndex, 
		pageIndex:this.currentPageIndex,
		tick: this.tick};
	this.pageProducer.addPrivateLocationData(loc);
	return loc;
};

PagedView_proto.setCurrentLocation = function(loc, doneFn)
{
	var prev = this.getCurrentPage();
	var self = this;
	this.resolveLocation(loc, function() {
		self.currentChunkIndex = loc.chunkIndex;
		self.currentPageIndex = loc.pageIndex;
		self.moveBackward(0, false, function(moved) {
			self.transitionToCurrentPage(prev);
			if(doneFn)
				doneFn();
		});
	});
};

PagedView_proto.resolveLocation = function(loc, doneFn)
{
	if( loc.chunkIndex === undef || loc.pageIndex === undef || loc.tick != this.tick )
	{
		var self = this;
		var pageProducer = this.pageProducer;
		pageProducer.resolveAndLoadLocationChunk(loc, function() {
			if( loc.chunkIndex === undef )
				loc.chunkIndex = 0;
			var maxPage = 0;
			while( !pageProducer.resolveLocationPage(loc) )
			{
				maxPage++;
				if( !self.formatUpToPage(loc.chunkIndex, maxPage) )
				{
					loc.pageIndex = 0;
					break;
				}
			}
			loc.tick = self.tick;
			doneFn();
		});
	}
	else
	{
		doneFn();
	}
};

PagedView_proto.transitionToCurrentPage = function(prevObj, opt)
{
    var pgt = this.pgt;
    if( pgt.isPrinting() )
        return;
    var currObj = this.getCurrentPage();
	this.setProxyVisibility(prevObj, false);
    if( this.stacking != "single" )
    {
    	// no transitions to speak of - all pages are always visible
    	this.setProxyVisibility(currObj, true);
        return;
    }
    var self = this;
    var handleProxyVisibility = function() {self.setProxyVisibility(currObj, true);};
    if( !this.pageTransition )
        this.pageTransition = pgt.isWebKit() ? "verticalSide" : "none";
    var curr = currObj.page;
    var prev = prevObj ? prevObj.page : null;
    var currPageStyle = curr.style;
    var prevPageStyle = (prev ? prev.style : null);
    if( this.pageTransition != "none" && prevPageStyle && opt && prev !== curr )
    {
        // fancier transition
        if( this.pageTransition == "verticalSlide" )
        {
            var distV = pgt.viewportHeight/pgt.units.rpx + 30;
            currPageStyle.WebkitTransition = "none";
            currPageStyle.visibility = "visible";
            currPageStyle.top = (opt=="forward"?distV:-distV) + "px";
            var completeV = function()
            {
                currPageStyle.WebkitTransition = "top 0.5s ease-in-out";
                prevPageStyle.WebkitTransition = "top 0.5s ease-in-out";
                currPageStyle.top = "0px";
                prevPageStyle.top = (opt=="forward"?-distV:distV) + "px";
                setTimeout(handleProxyVisibility, 500);
            };
            pgt.scheduleCall(completeV);
        }
        else
        {
            // horizontal
            var distH = pgt.getViewportWidth()/pgt.units.rpx + 30;
            currPageStyle.WebkitTransition = "none";
            currPageStyle.visibility = "visible";
            currPageStyle.left = (opt=="forward"?distH:-distH) + "px";
            var completeH = function()
            {
                currPageStyle.WebkitTransition = "left 0.5s ease-in-out";
                prevPageStyle.WebkitTransition = "left 0.5s ease-in-out";
                currPageStyle.left = "0px";
                prevPageStyle.left = (opt=="forward"?-distH:distH) + "px";
                setTimeout(handleProxyVisibility, 500);
            };
            pgt.scheduleCall(completeH);
        }
        return;      
    }
    
    // default transition: just replace one page with another
    currPageStyle.WebkitTransition = "none";
    currPageStyle.top = "0px";
    currPageStyle.left = "0px";
    if( prevPageStyle )
        prevPageStyle.visibility = "hidden";
    currPageStyle.visibility = "visible";
    handleProxyVisibility();
};

PagedView_proto.setProxyVisibility = function(pageObj, visible)
{
	if( !pageObj )
		return;
    var proxyList = pageObj.proxyList;
    if( !proxyList )
        return;
    for( var i = 0 ; i < proxyList.length ; i++ )
    {
        var proxyRecord = proxyList[i];
        var overlay = proxyRecord.target;
        var overlayStyle = overlay.style;
        if( !visible )
        {
            overlayStyle.visibility = "hidden";
            continue;
        }
        if( !overlay.parentNode )
        {
            if( !this.overlayLayer )
                this.buildOverlayLayer();
            this.overlayLayer.appendChild(overlay);
            overlayStyle.margin = "0px";
        }
        overlayStyle.position = "absolute";
        var rect = this.pgt.getBoundingClientRect(proxyRecord.view);
        overlayStyle.left = rect.left + "px";
        overlayStyle.top = rect.top + "px";
        overlayStyle.display = "block";
        overlayStyle.visibility = "visible";
     }
};

PagedView_proto.formatUpToPage = function(chunkIndex, pageIndex)
{
	var chunk = this.getChunk(chunkIndex);
	var pp = this.pageProducer;
	var fitChunk = pp.needChunkFitting(chunkIndex);
    while( true )
    {
    	if( chunk.done )
    		return pageIndex < chunk.pages.length;
    	if( !fitChunk && pageIndex < chunk.pages.length )
    		return true;
        this.privateBuildNextPageInChunk(chunkIndex);
        if( fitChunk && chunk.done )
        {
        	// set # of pages
        	var pageCount = pp.getChunkFittedPages();
        	this.clearChunkPages(chunk, pageCount);
        }
    }
};

PagedView_proto.moveForward = function(count, preloadOnly, doneFn, chunkIndex, pageIndex)
{
	if( chunkIndex === undef )
	{
		chunkIndex = this.currentChunkIndex;
		pageIndex = this.currentPageIndex;
	}
	var chunkCount = this.pageProducer.getChunkCount();
	do
	{
		if( !this.pageProducer.chunkReady(chunkIndex) )
		{
			var self = this;
			this.pageProducer.loadChunk(chunkIndex, function() {
				self.moveForward(count, preloadOnly, doneFn, chunkIndex, pageIndex);
			});
			return;
		}
		if( this.formatUpToPage(chunkIndex, pageIndex) )
		{
			if( !preloadOnly )
			{
				this.currentChunkIndex = chunkIndex;
				this.currentPageIndex = pageIndex;
			}
			if( count == 0 )
				break;
			count--;
			pageIndex++;
		}
		else
		{
			chunkIndex++;
			pageIndex = 0;
		}
	}
	while( chunkIndex < chunkCount );
	doneFn(count == 0 && chunkIndex < chunkCount);
};

PagedView_proto.moveBackward = function(count, preloadOnly, doneFn, chunkIndex, pageIndex)
{
	if( chunkIndex === undef )
	{
		chunkIndex = this.currentChunkIndex;
		pageIndex = this.currentPageIndex;
	}
	do
	{
		if( !this.pageProducer.chunkReady(chunkIndex) )
		{
			var self = this;
			this.pageProducer.loadChunk(chunkIndex, function() {
				self.moveBackward(count, preloadOnly, doneFn, chunkIndex, pageIndex);
			});
			return;
		}
		if( !this.formatUpToPage(chunkIndex, pageIndex) )
		{
			var chunk = this.getChunk(chunkIndex);
			pageIndex = chunk.pages.length - 1;
		}
		if( pageIndex >= 0 )
		{
			if( count < pageIndex )
			{
				pageIndex -= count;
				count = 0;
			}
			else
			{
				count -= pageIndex;
				pageIndex = 0;
			}
			if( !preloadOnly )
			{
				this.currentChunkIndex = chunkIndex;
				this.currentPageIndex = pageIndex;
			}
			if( count == 0 )
				break;
			count--;
		}
		chunkIndex--;
		pageIndex = Number.POSITIVE_INFINITY;
	}
	while( chunkIndex >= 0 );
	doneFn(count == 0 && chunkIndex >= 0);
};

PagedView_proto.firstPage = function(doneFn)
{
	var prev = this.getCurrentPage();
	var self = this;
	this.currentChunkIndex = 0;
	this.currentPageIndex = 0;
	this.moveForward(0, false, function(moved) {
		self.transitionToCurrentPage(prev);
		if( doneFn )
			doneFn(moved);
	});
};

PagedView_proto.lastPage = function(doneFn)
{
	var prev = this.getCurrentPage();
	var self = this;
	this.currentChunkIndex = this.pageProducer.getChunkCount()-1;
	this.currentPageIndex = Number.POSITIVE_INFINITY;
	this.moveBackward(0, false, function(moved) {
		self.transitionToCurrentPage(prev);
		if( doneFn )
			doneFn(moved);
	});
};

PagedView_proto.nextPage = function(doneFn, n, preloadOnly)
{
	n = n || 1;
	var prev = this.getCurrentPage();
	var self = this;
	var res = "pending";
	this.moveForward(n, preloadOnly, function(moved) {
		if( moved && !preloadOnly )
			self.transitionToCurrentPage(prev, "forward");
		res = moved ? "moved" : "end";
		if( doneFn )
			doneFn(moved);
	});
	return res;
};

PagedView_proto.previousPage = function(doneFn, n, preloadOnly)
{
	n = n || 1;
	var prev = this.getCurrentPage();
	var self = this;
	var res = "pending";
	this.moveBackward(n, preloadOnly, function(moved) {
		if( moved && !preloadOnly )
			self.transitionToCurrentPage(prev, "backward");
		res = moved ? "moved" : "end";
		if( doneFn )
			doneFn(moved);
	});
	return res;
};


})(pgt);