// Page Template Library
// parse and apply custom CSS styles

// begin private scope
(function(pgt, undef) {

if( !pgt )
    alert("pgt-util.js is required");
pgt.provides("pgt-style");

var privateToJSName = function(name)
{
    if( name == "float" )
        return "cssFloat";
    var res = "";
    var k = 0;
    var len = name.length;
    while(true)
    {
        var n = name.indexOf("-",k);
        if( n < 0 )
        {
            res += name.substr(k);
            break;
        }
        res += name.substring(k, n);
        n++;
        if( n > len )
            break;
        res += name.substr(n,1).toUpperCase();
        k = n+1;
    }
    return res;
};

var privateSetCSSProperty = function(style, name, val)
{
    if( style.setProperty )
    {
        style.setProperty(name, val, "");
        var v = style.getPropertyValue(name);
        return v != "" && v !== null;
    }
    else
    {
        var jsname = privateToJSName(name);
        if( style[jsname] === undef )
            return false;
        style[jsname] = val;
        return true;
    }
};

var specificityAndOrder = function(r1, r2)
{
    var d = r1.specificity - r2.specificity;
    if( d === 0 )
        d = r1.order - r2.order;
    return d;
};

var privateApplyRulesToElement = function( style, elem, rules, setter )
{
    rules.sort(specificityAndOrder);
    for( var j = 0 ; j < rules.length ; j++ )
    {
        var rule = rules[j];
        if( !rule.cond || rule.cond(elem) )
        {
            for( var name in rule.props )
            {
                var val = style.calc(rule.props[name]);
                if( !setter || !setter(elem.style, name, val) )
                    elem.pgtstyle[name] = val;
            }
        }
    }
};

var privateApplyImplicitRules = function( pgtstyle, nsuri, localName )
{
    if( nsuri == "http://www.w3.org/2000/svg" )
        pgtstyle["break-inside"] = "avoid";
    if( nsuri == "http://www.w3.org/1999/xhtml" )
    {
        switch( localName )
        {
        case "table" :
        case "img" :
        case "object" :
        case "embed" :
        case "iframe" :
        case "canvas" :
        case "audio" :
        case "video" :
        case "form" :
        case "li" :
            pgtstyle["break-inside"] = "avoid";
            break;
        }
    }
};

var privateApplyToElement = function( self, elem, pgtonly, isRoot )
{
    self.pgt.clearViews(elem);
    var localName = (isRoot ? "body" : elem.localName);
    var rules = self.tags[localName];
    var classStr = elem.getAttribute("class");
    if( classStr != null && classStr != "" )
    {
        var classList = classStr.split(/\s+/);
        for( var i = 0 ; i < classList.length ; i++ )
        {
            var classRules = self.classes[classList[i]];
            if( classRules )
            {
                if( rules )
                    rules = rules.concat( classRules );
                else
                    rules = classRules;
            }
        }
    }
    if( rules )
        privateApplyRulesToElement( self, elem, rules, (pgtonly?null:privateSetCSSProperty) );
    privateApplyImplicitRules( elem.pgtstyle, elem.namespaceURI, localName );
};

var needsProxy = function(elem)
{
    if( elem.pgtstyle["view-placement"] == "overlay" )
        return true;
    var localName = elem.localName;
    switch( localName )
    {
    case "iframe" :
    case "video" :
    case "audio" :
    case "object" :
    case "embed" :
            return true;
    }
    return false;
};

var empty = function() {};

var Styler = function(owner, root, pgtonly, startOffset)
{
    this.owner = owner;
    this.pgt = owner.pgt;
    this.root = root;
    this.pgtonly = pgtonly;
    this.startOffset = startOffset ? startOffset : 0;
    this.postStylingHook = empty;
    this.reset();
};

var Styler_proto = Styler.prototype;

Styler_proto.reset = function()
{
    this.curr = this.root;
    this.done = false;
    this.offset = this.startOffset;
    this.insideOverlay = false;
    this.curr.pgtstyle = {};
};

Styler_proto.setPostStylingHook = function(postStylingHook)
{
    this.postStylingHook = postStylingHook;
};

Styler_proto.isDone = function()
{
    return this.root == this.curr && this.done;
};

Styler_proto.step = function()
{
    // *** find next current element ***
    var elem = this.curr;
    while( true )
    {
        if( this.done )
        {
            elem.pgtstyle.endOffset = this.offset;
            if( elem == this.root )
            {
                this.curr = elem;
                return true; // done
            }
            if( elem.pgtProxy )
            {
                this.insideOverlay = false;
                elem = elem.pgtProxy;
                continue;
            }
            var next = elem.nextSibling;
            if( next )
            {
                elem = next;
                this.done = false;
            }
            else
            {
                elem = elem.parentNode;
                continue;
            }
        }
                
        elem.pgtstyle = {startOffset: this.offset, styler:this};
        switch( elem.nodeType )
        {
        case 3 :
        case 4 :
        case 5 :
            this.offset += elem.data.length;
        case 2 :
        case 6 :
        case 7 :
        case 8 :
            elem.pgtstyle.endOffset = this.offset;
            this.done = true;
            continue;
        }

        this.curr = elem;
        break;
    }
    
    // *** apply styles to the current element ***
    privateApplyToElement(this.owner, elem, this.pgtonly, elem == this.root);

    // *** handle proxies ***
    var proxy;
    if( elem.pgtProxyTarget )
    {
        // elem is a previously-created proxy
        proxy = elem;
        elem = elem.pgtProxyTarget;
        this.curr = elem;
        elem.pgtstyle = {startOffset: this.offset, styler:this};
        privateApplyToElement(this.owner, elem, this.pgtonly, elem == this.root);
    }
    if( !this.insideOverlay && needsProxy(elem) )
    {
        // create proxy if needed
        if( !proxy )
        {
        	var elemStyle = elem.getAttribute("style");
            proxy = elem.ownerDocument.createElementNS(pgt.ns.xhtml, "div");
            proxy.className = elem.className;
            if( elemStyle )
            	proxy.setAttribute("style", elemStyle);
            proxy.pgtstyle = {startOffset: this.offset, styler:this};
            privateApplyToElement(this.owner, proxy, this.pgtonly, elem == this.root);
            proxy.pgtProxyTarget = elem;
            elem.parentNode.replaceChild(proxy, elem);
            elem.pgtSavedStyle = elemStyle;
            elem.pgtProxy = proxy;
        }
        proxy.style.visibility = "hidden";
        proxy.pgtstyle["break-inside"] = "avoid";
        this.insideOverlay = true;
    }
    else if( proxy )
    {
        // destroy a proxy that is no longer applicable
        proxy.parentNode.replaceChild(elem, proxy);     
        elem.setAttribute( "style", elem.pgtSavedStyle );
        delete elem.pgtProxy;
    }
    
    this.postStylingHook(elem, elem == this.root);
    
    // calculate offset and style children
    var child = elem.firstChild;
    if( child )
        this.curr = child;
    else
        this.done = true;
    return false;
};

Styler_proto.ensureOffset = function(offset)
{
    while( this.offset <= offset )
        if( this.step() )
            return;
};

Styler_proto.ensureNode = function(node)
{
    while( true )
    {
        var pgtstyle = node.pgtstyle;
        if( pgtstyle )
        {
            if( pgtstyle.styler === this )
                break;
        }
        if( this.step() )
            return;
    }
};

pgt.Style = function(pgt)
{
    if( pgt )
        this.protectedInit(pgt);
};

pgt.createStyle = function()
{
    return new this.Style(this);
};

var Style_proto = pgt.Style.prototype;

Style_proto.protectedInit = function(pgt)
{
    this.pgt = pgt;
    this.classes = {};
    this.tags = {};
    this.order = 1;
    this.fontFaces = [];
};

Style_proto.calc = function(val)
{
    return val;
};

Style_proto.parseValue = function(text, propName)
{
    return text;
};

Style_proto.parseAtRule = function(ruleName, ruleHead, ruleBody, cond)
{
    switch( ruleName )
    {
    case "font-face" :
        this.privateParseFontFace(ruleBody, cond);
        break;
    case "pgt-params" :
        this.privateParseParams(ruleBody);
        break;
    }
};

Style_proto.privateParseFontFace = function(body, cond)
{
    if( cond !== null )
        this.pgt.error("no @font-face inside conditionals!");
    else
    {
        var props = this.parseProperties(body, cond);
        this.fontFaces.push(props);
    }
};

Style_proto.privateParseParams = function(body)
{
    this.params = this.parseProperties(body);
};

Style_proto.getParam = function(name)
{
    if( this.params )
        return this.params[name];
    return null;
};

pgt.applyViewStyles = function(view, master)
{
    var pgtstyle = master.pgtstyle;
    if( pgtstyle )
    {
        var style = view.style;
        for( var name in pgtstyle )
            privateSetCSSProperty(style, name, pgtstyle[name]);
    }
};

Style_proto.createStyler = function( root, pgtonly, offset )
{
    return new Styler( this, root, pgtonly, offset );
};

Style_proto.getApplyViewStylesFunc = function()
{
    if( !this.applyViewStyleFunc )
    {
        var self = this;
        this.applyViewStyleFunc = function(view, orig)
        { 
            self.pgt.applyViewStyles(view, orig);
            privateApplyToElement(self, view);
        };
    }
    return this.applyViewStyleFunc;
};

var propPatt = /^\s*([\w\-]+)\s*:\s*([^;]*);?/;
var selectorRulePatt = /^\s*([^\{;]*[^{;\s])\s*\{/;
var atRulePatt = /^\s*@([\w\-]+)\s*([^\{;]*[^{;\s])?\s*([\{;])/;
var wsPatt = /^\s+/;
var commentPatt = /^\s*\/\*([^\*]|(\*+([^\/\*])))*\*+\/|\/\/[^\n\r]*/;

var findRuleBody = function(self, text, bodyStart)
{
    var next;
    var count = 0;
    var len = text.length;
    for( next = bodyStart ; next < len ; next++ )
    {
        var c = text.charCodeAt(next);
        if( c == 123 ) // '}'
            count++;
        else if( c == 125 )
        {
            count--;
            if( count < 0 )
                break;
        }
    }
    if( count >= 0 )
    {
        self.pgt.error( "could not parse: " + text );
        return -1; // syntax error
    }
    return next;
};

var addStyleToStore = function( store, key, props, cond, order, specificity )
{
    var s = store[key];
    if( s == null )
    {
        s = [];
        store[key] = s;
    }
    s[s.length] = {cond: cond, props: props, order: order, specificity:specificity};
};

var insetIndices = [
    [0, 0, 0, 0],
    [0, 1, 0, 1],
    [0, 1, 2, 1],
    [0, 1, 2, 3]
];

var insetsShorthand = function(style, dict, name, value)
{
    var arr = value.split(/\s+/);
    if( arr[arr.length-1] == "" )
        arr.pop();
    var len = arr.length;
    if( len == 0 || len > 4 )
        return; // syntax error
    var index = name.indexOf("-");
    if( index < 0 )
        index = name.length;
    var prefix = name.substr(0,index);
    var suffix = name.substr(index);
    var ind = insetIndices[len-1];
    style.privateProcessProperty(dict, prefix + "-top" + suffix, arr[ind[0]]);
    style.privateProcessProperty(dict, prefix + "-right" + suffix, arr[ind[1]]);
    style.privateProcessProperty(dict, prefix + "-bottom" + suffix, arr[ind[2]]);
    style.privateProcessProperty(dict, prefix + "-left" + suffix, arr[ind[3]]);
};

var borderShorthand = function(style, dict, name, value)
{
    style.privateProcessProperty(dict, name + "-top", value);
    style.privateProcessProperty(dict, name + "-right", value);
    style.privateProcessProperty(dict, name + "-bottom", value);
    style.privateProcessProperty(dict, name + "-left", value);
};

var borderComponentShorthand = function(style, dict, name, value)
{
    var arr = value.split(/\s+/);
    if( arr[arr.length-1] == "" )
        arr.pop();
    var len = arr.length;
    if( len == 0 || len > 3 )
        return; // syntax error
    var bs = "solid";
    var color = "black"; // should be current color...
    var width = "2px";
    for( var i = 0 ; i < arr.length ; i++ )
    {
        var c = arr[i];
        switch( c )
        {
            case "thin" :
                width = "1px";
                break;
            case "medium" :
                width = "3px";
                break;
            case "thick" :
                width = "6px";
                break;
            case "solid" :
            case "none" :
            case "hidden" :
            case "dotted" :
            case "dashed" :
            case "double" :
            case "groove" :
            case "ridge" :
            case "inset" :
            case "outset" :
                bs = c;
                break;
            default:
                if( c.match( style.pgt.patterns.cssLength ) )
                    width = c;
                else
                    color = c;
        }
    }
    style.privateProcessProperty(dict, name + "-width", width);
    style.privateProcessProperty(dict, name + "-style", bs);
    style.privateProcessProperty(dict, name + "-color", color);    
};

var columnsShorthand = function(style, dict, name, value)
{
    var arr = value.split(/\s+/);
    if( arr[arr.length-1] == "" )
        arr.pop();
    style.privateProcessProperty(dict, "column-count", "auto" );
    style.privateProcessProperty(dict, "column-width", "auto" );
    for( var i = 0 ; i < arr.length ; i++ )
    {
        var v = arr[i];
        if( v.match( /\d+/ ) )
            style.privateProcessProperty(dict, "column-count", v );
        else if( v != "auto" )
            style.privateProcessProperty(dict, "column-width", v );
    }
};

var makeShothands = function()
{
    var shorthands = {};
    shorthands.margin = insetsShorthand;
    shorthands.padding = insetsShorthand;
    shorthands["border-width"] = insetsShorthand;
    shorthands.border = borderShorthand;
    shorthands["border-top"] = borderComponentShorthand;
    shorthands["border-left"] = borderComponentShorthand;
    shorthands["border-right"] = borderComponentShorthand;
    shorthands["border-bottom"] = borderComponentShorthand;
    shorthands.columns = columnsShorthand;
    shorthands["column-rule"] = borderComponentShorthand;
    //shorthands.font = fontShorthand;
    //shorthands.background = backgroundShorthand;
    return shorthands;
};

var shorthands = makeShothands();

Style_proto.privateProcessProperty = function(propDict, propName, propValue)
{
    var shorthandProc = shorthands[propName];
    if( shorthandProc )
        shorthandProc(this, propDict, propName, propValue);
    else
    {
        propValue = this.parseValue(propValue, propName);
        propDict[propName] = propValue;
    }
};

Style_proto.parseProperties = function(text, cond)
{
    var res = {};
    while( true )
    {
        var r;
        var next;
        if( (r = text.match(propPatt)) != null )
        {
            var propName = r[1];
            var propValue = r[2];
            this.privateProcessProperty(res, propName, propValue);
            next = r[0].length;
        }
        else if( (r = text.match(atRulePatt)) != null )
            next = this.privateProcessAtRule( text, r, cond );
        else if( (r = text.match(commentPatt)) != null )            
            next = r[0].length; // skip comment
        else if( (r = text.match(wsPatt)) != null )            
            next = r[0].length; // just whitespace
        else
            break; // syntax error
        text = text.substr(next);
    }
    return res;
};

Style_proto.privateProcessAtRule = function( text, r, cond )
{
    var ruleName = r[1];
    var ruleHead = r[2];
    var ruleBodyChar = r[3];
    var ruleBody;
    var atBodyStart = r[0].length;
    var next;
    if( ruleBodyChar == "{" )
    {
        next = findRuleBody(this, text, atBodyStart);
        if( next < 0 )
        {
            this.pgt.error( "could not parse: " + text );
            return text.length; // error
        }
        ruleBody = text.substring(atBodyStart, next);
        next++; // skip '}'
    }
    else
        next = atBodyStart;
    this.parseAtRule( ruleName, ruleHead, ruleBody, cond );
    return next;
};

Style_proto.privateProcessSelectorRule = function( text, r, cond )
{
    var selectorsStr = r[1];
    var ruleBodyStart = r[0].length;
    var next = findRuleBody(this, text, ruleBodyStart);
    if( next < 0 )
    {
        this.pgt.error( "could not parse: " + text );
        return text.length; // error
    }
    var ruleBody = text.substring(ruleBodyStart, next);
    next++; // skip '}'
    if( !this.atOnly )
    {
        var ruleProps = this.parseProperties(ruleBody, cond);
        var selectors = selectorsStr.split( /\s*,\s*/ );
        for( var i = 0 ; i < selectors.length ; i++ )
        {
            var selector = selectors[i];
            if( selector.match( /\.[\w\-]+/ ) )
            {
                var className = selector.substr(1);
                addStyleToStore( this.classes, className, ruleProps, cond, this.order++, 0x100 );                
            }
            else if( selector.match( /[\w\-]+/ ) )
            {
                addStyleToStore( this.tags, selector, ruleProps, cond, this.order++, 1 );
            }
            else
            {
                // TODO: plug jQuery here
                this.pgt.error("unsupported selector: " + selector );
            }
        }
    }
    return next;
};

Style_proto.parseStylesheet = function(text, cond, url)
{
    var savedURL, ruleBody, className, tagName;
    if( url != null )
    {
        savedURL = this.baseURL;
        this.baseURL = url;
    }
    while( text != "" )
    {
        var r;
        var next;
        if( (r = text.match(atRulePatt)) != null )
            next = this.privateProcessAtRule( text, r, cond );
        else if( (r = text.match(commentPatt)) != null )            
            next = r[0].length; // skip comment
        else if( (r = text.match(selectorRulePatt)) != null )
            next = this.privateProcessSelectorRule( text, r, cond );
        else if( (r = text.match(wsPatt)) != null )            
            next = r[0].length; // just whitespace
        else
        {
            this.pgt.error( "could not parse: " + text );
            break; // syntax error
        }
        text = text.substr(next);
    }
    if( url != null )
        this.baseURL = savedURL;
};

Style_proto.parseStylesheetHandler = function()
{
    var self = this;
    return function(text, url)
    {
        var saveAtOnly = self.atOnly;
        var pgt = self.pgt;
        pgt.trace("parsing stylesheet " + url );
        pgt.startTimer("parseStylesheet");
        self.atOnly = true;
        self.parseStylesheet(text, null, url);
        self.atOnly = saveAtOnly;
        pgt.stopTimer("parseStylesheet");
    };
};

Style_proto.parsePGTStylesheetHandler = function()
{
    var self = this;
    return function(text, url)
    {
        var pgt = self.pgt;
        pgt.trace("parsing stylesheet " + url );
        pgt.startTimer("parseStylesheet");
        self.parseStylesheet(text, null, url);
        pgt.stopTimer("parseStylesheet");
    };
};

Style_proto.parseStyleElements = function(doc)
{
    var pgt = this.pgt;
    var list = doc.getElementsByTagName("style");
    var arr = [];
    var url = doc.URL;
    for( var i = 0 ; i < list.length ; i++ )
    {
        pgt.startTimer("parseStylesheet");
        var saveAtOnly = this.atOnly;
        this.parseStylesheet(list[i].textContent, null, url);
        this.atOnly = saveAtOnly;
        pgt.stopTimer("parseStylesheet");
    }
};

pgt.findLinks = function(srcDoc, relPatt)
{
    var list = srcDoc.getElementsByTagName("link");
    var arr = [];
    for( var i = 0 ; i < list.length ; i++ )
    {
        var e = list[i];
        var rel = e.getAttribute("rel") + "";
        if( rel.match(relPatt) && e.getAttribute("type") == "text/css" )
        {
            var src = e.getAttribute("href");
            if( src && src != "" )
            {
                src = pgt.resolveURL(src, srcDoc.URL);
                arr.push({src:src, rel:rel});
            }
        }
    }
    return arr;
};

var checkFont = function(pgt, props, envElem)
{
    var doc = envElem.ownerDocument;
    var probe = doc.createElement("div");
    probe.style.position = "absolute";
    probe.style.lineHeight = "1";
    probe.style.visibility = "hidden";
    
    envElem.appendChild(probe);
    var span = doc.createElement("span");
    probe.appendChild(span);
    span.appendChild(doc.createTextNode("M"));
    
    for( var name in props )
    {
        if( !name.match(/^font-/) )
            continue;
        var val = props[name];
        privateSetCSSProperty(probe.style, name, val);
    }
    
    probe.style.fontSize = "144pt";
    probe.style.fontFamily = props["font-family"] + ", monospace";
    var r = span.getBoundingClientRect();
    var w = r.width;
    var h = r.height;
        
    probe.style.fontFamily = "monospace";
    r = span.getBoundingClientRect();
    
    var haveFont = r.width != w || r.height != h;
        
    envElem.removeChild(probe);
    
    return haveFont;
};

Style_proto.insertFonts = function()
{
    var pgt = this.pgt;
    var stylesheet = "";
    var fontFaces = this.fontFaces;
    var len = fontFaces.length;
    for( var i = 0 ; i < len ; i++ )
    {
        stylesheet += "@font-face {\n";
        var props = fontFaces[i];
        for( var name in props )
        {
            var val = props[name];
            stylesheet += name;
            stylesheet += ": ";
            stylesheet += val;
            stylesheet += ";\n";
        }
        stylesheet += "}\n";        
    }
    if( stylesheet.length > 0 )
    {
        var doc = pgt.document;
        var style = doc.createElement("style");
        style.setAttribute("type", "text/css");
        style.appendChild(doc.createTextNode(stylesheet));
        doc.head.appendChild(style);      
    }
};

Style_proto.waitForFonts = function(maxDelay, container)
{
    var pgt = this.pgt;
    if( !pgt.isWebKit() ) // only WebKit can load fonts
        return;

    if( maxDelay == null )
        maxDelay = 1000;
    if( container == null )
        container = pgt.document.body;
        
    var fontFaces = this.fontFaces;
    var index = 0;
    var totalDelay = 0;

    var checkNextFont = function()
    {
        while( index < fontFaces.length && checkFont(pgt, fontFaces[index], container) )
            index++;
                
        if( index >= fontFaces.length || totalDelay > maxDelay )
            pgt.taskDone();
        else
        {
            setTimeout(checkNextFont, 100);
            totalDelay += 100;
        }
    };
    
    pgt.scheduleTask( checkNextFont );
};



})(pgt);