// Page Template Library
// page templates

// begin private scope
(function(pgt, undef) {

if( !pgt )
    alert("pgt-util.js is required");
pgt.provides("pgt-templ");
pgt.requires("pgt-style");
pgt.requires("pgt-flow");
pgt.requires("pgt-expr");
pgt.requires("pgt-dstyle");
pgt.requires("pgt-fit");
pgt.requires("pgt-geom");

pgt.Template = function(pgt)
{
    if( pgt )
        this.protectedInit(pgt);
};

pgt.createTemplate = function()
{
    return new this.Template(this);
};

var Template_proto = new pgt.DynamicStyle();
pgt.Template.prototype = Template_proto;
Template_proto.constructor = pgt.Template;

Template_proto.protectedInit = function(pgt)
{
    pgt.DynamicStyle.prototype.protectedInit.call(this, pgt);
    this.pageMasters = [];
    this.currentPage = 0;
    this.templateScope = new TemplateScope(pgt, this);
    this.scope = this.templateScope;
    this.templateResolveQueue = [];
    this.resolveQueue = this.templateResolveQueue;
    this.modes = {};
    this.fitters = {};
    this.pgtDepth++;
    this.privateParsePageMaster( "@region body{column-width:20em;margin:1em;}" );
    this.pgtDepth--;
};

Template_proto.needGlobalFitting = function()
{
    return this.globalFitter !== undef;
};

Template_proto.getTargetPages = function()
{
    if( this.globalFitter )
        return this.globalFitter.getTargetPages();
    return null;
};

Template_proto.parseAtRule = function(ruleName, ruleHead, ruleBody, cond)
{
    switch( ruleName )
    {
    case "page-master" :
        this.pgtDepth++;
        this.privateParsePageMaster(ruleBody, cond);
        this.pgtDepth--;
        break;
    case "region" :
        this.privateParseRegion(ruleHead, ruleBody, cond);
        break;
    case "mode" :
        this.privateParseMode(ruleHead, ruleBody, cond);
        break;
    case "flow" :
        this.privateParseFlow(ruleHead, ruleBody, cond);
        break;    
    default:
        pgt.DynamicStyle.prototype.parseAtRule.call( this, ruleName, ruleHead, ruleBody, cond );
    }
};

Template_proto.privateParsePageMaster = function(ruleBody, cond)
{
    var pageMaster = new PageMaster(this.pgt, this, cond);
    var saveScope = this.scope;
    var saveResolveQueue = this.resolveQueue;
    var savePageMaster = this.currentPageMaster;
    this.scope = pageMaster.scope;
    this.resolveQueue = pageMaster.resolveQueue;
    this.currentPageMaster = pageMaster;
    var props = this.parseProperties( ruleBody, cond );
    this.scope = saveScope;
    this.resolveQueue = saveResolveQueue;
    this.currentPageMaster = savePageMaster;
    pageMaster.setProperties(props);
    var pms = this.pageMasters;
    if( pms.length > 0 )
        pms.splice(pms.length-1, 0, pageMaster);
    else
        pms.push(pageMaster);
};

Template_proto.privateParseRegion = function(name, ruleBody, cond)
{
    var props = this.parseProperties( ruleBody, cond );
    this.currentPageMaster.addRegion( name, props, cond );
};

Template_proto.privateParseMode = function(name, body, cond)
{
    var style = this.modes[name];
    if( style === undef )
    {
        style = this.pgt.createDynamicStyle();
        this.modes[name] = style;
    }
    style.parseStylesheet(body, cond, this.baseURL);
};

Template_proto.privateParseFlow = function(name, body, cond)
{
    var props = this.parseProperties( body, cond );
    if( props["fitting-target"] )
    {
        var fitter = this.pgt.createTextFitter(this, props, name);
        this.fitters[name] = fitter;
        if( name == "body" )
            this.globalFitter = fitter;
    }
};

Template_proto.resolve = function()
{
    var show = this.getParam("show");
    if( show )
    {
        this.showGrid = show.match(/\bgrid\b/) != null;
        this.showFragments = show.match(/\bfragments\b/) != null;
        this.showRegions = show.match(/\bregions\b/) != null;
        this.showRegionMargins = show.match(/\bregion-margins\b/) != null;
    }
    this.scope = this.templateScope;
    this.resolveQueue = this.templateResolveQueue;
    pgt.DynamicStyle.prototype.resolve.call(this);
    for( var m in this.modes )
        this.modes[m].resolve();    
    var pms = this.pageMasters;
    for( var i = 0 ; i < pms.length ; i++ )
        pms[i].resolve();
};

Template_proto.prepare = function(context)
{
    if( !context )
        context = this.pgt.createExprContext();   
    this.context = context;
    for( var m in this.modes )
        this.modes[m].prepare(context);
    for( var f in this.fitters )
        this.fitters[f].prepare(context);
};

Template_proto.reset = function()
{
    this.currentPage = 0;
};

Template_proto.layoutPage = function(pageContainer, doc)
{
    var pgt = this.pgt;
    pgt.startTimer("layoutPage");
    this.currentPage++;
    this.currentPageMaster = null;
    this.prepare();
    var pageMasters = this.pageMasters;
    var docOffset = doc.getCurrentBodyOffset();
    var lookupOffset;
    for( var i = 0 ; i < pageMasters.length ; i++ )
    {
        var pageMaster = pageMasters[i];
        lookupOffset = pageMaster.getLookupOffset();
        doc.setLookupOffset(docOffset + lookupOffset);
        pageMaster.prepare(doc);
        if( pageMaster.enabled )
        {
            this.currentPageMaster = pageMaster;
            break;
        }
    }
    if( this.currentPageMaster )
    {
        this.prepare();
        doc.beginPage();
        this.currentPageMaster.layoutPage(pageContainer, doc);
        doc.endPage();
    }
    var bodyFitter = this.globalFitter;
    var hasMore = doc.hasMoreBodyContent();
    if( bodyFitter )
    {
        var targetPages = bodyFitter.getTargetPages();
        if( hasMore )
        {
            if( this.currentPage == targetPages )
                bodyFitter.targetReached();
        }
        else
        {
            if( !targetPages || targetPages >= this.currentPage )
                bodyFitter.targetReached();
        }
    }
    if( !hasMore )
    {
        for( var f in this.fitters )
        {
            if( f != "body" )
                this.fitters[f].targetReached();
            this.fitters[f].layoutDone();
        }   
    }
    pgt.stopTimer("layoutPage");
    pgt.status("lookupSize", lookupOffset);
    pgt.status("consumedSize", doc.getCurrentBodyOffset() - docOffset);
    return hasMore;
};

Template_proto.prefit = function()
{    
    for( var flowName in this.fitters )
    {
        var fitter = this.fitters[flowName];
        fitter.reset(true);
    }
};

Template_proto.fit = function(doc)
{    
    for( var flowName in this.fitters )
    {
        var fitter = this.fitters[flowName];
        fitter.reset(false);
        fitter.fit( doc.getFlow(flowName) );
    }
};

Template_proto.createPageProducer = function(doc)
{
	doc.setStyle(this);
	var pageProducer = new TemplatePageProducer(this.pgt, this, doc);
	pageProducer.reset();
	return pageProducer;
};

var TemplatePageProducer = function(pgt, template, doc)
{
	this.pgt = pgt;
	this.template = template;
	this.doc = doc;
	this.idmap = {};
};

var TemplatePageProducer_proto = TemplatePageProducer.prototype;

TemplatePageProducer_proto.reset = function()
{
	if( this.needChunkFitting() )
        this.template.prefit();		
	this.doc.resetFlows();
	this.idmap = {};
	this.template.reset();
	this.template.prepare();
	this.done = false;
};

TemplatePageProducer_proto.getChunkCount = function()
{
	return 1;
};

TemplatePageProducer_proto.chunkReady = function(chunkIndex)
{
	return true;
};

TemplatePageProducer_proto.loadChunk = function(chunkIndex, doneFn)
{
	doneFn();
};

TemplatePageProducer_proto.resolveAndLoadLocationChunk = function(loc, doneFn)
{
	this.loadChunk(loc.chunkIndex, doneFn);
};

TemplatePageProducer_proto.needChunkFitting = function(chunkIndex)
{
	return this.template.needGlobalFitting();
};

TemplatePageProducer_proto.getChunkFittedPages = function()
{
	return this.template.getTargetPages();
};

TemplatePageProducer_proto.buildNextPage = function(chunkIndex, pageIndex, pageObj)
{
	var currentPage = this.template.currentPage;
	if( chunkIndex != 0 || currentPage != pageIndex )
		this.pgt.error("bad state in TemplatePageProducer");
	if( this.done )
		return false;
	var doc = this.doc;
	doc.setViewTracker(new TemplateViewTracker(this, pageIndex, pageObj));	
	if( !doc.layoutPage(pageObj.page) )
	{
		if( this.needChunkFitting() )
		{
			doc.resetFlows();
			doc.setLookupOffset(Number.MAX_VALUE);
	        this.template.fit(this.doc);
		}
		this.done = true;
	}
	doc.setViewTracker(null);
	return true;
};

TemplatePageProducer_proto.addPrivateLocationData = function(loc)
{
	if( loc.pageIndex !== undef )
	{
		// TODO: add reference to the beginning of the page
	}
};

TemplatePageProducer_proto.resolveLocationPage = function(loc)
{
	var bookmark = loc.bookmark;
	if( bookmark )
	{
		if( bookmark.match(this.pgt.patterns.idFragment) )
		{
			var id = loc.bookmark.substr(1);
			var ref = this.idmap[id];
			if( !ref )
				return false;
			loc.chunkIndex = 0;
			loc.pageIndex = ref.pageIndex;
			return true;
		}
	}
	return true;
};

var TemplateViewTracker = function(pageProducer, pageIndex, pageObj)
{
	this.pgt = pageProducer.pgt;
	this.pageProducer = pageProducer;
	this.idmap = pageProducer.idmap;
	this.pageIndex = pageIndex;
	this.pageObj = pageObj;
};

var TemplateViewTracker_proto = TemplateViewTracker.prototype;

TemplateViewTracker_proto.reportId = function(id)
{
	if( !this.idmap[id] )
		this.idmap[id] = {pageIndex: this.pageIndex};
};

TemplateViewTracker_proto.reportProxy = function(master, view, proxyTarget)
{
	var pageObj = this.pageObj;
    var proxyList = pageObj.proxyList;
    if( !proxyList )
    {
        proxyList = [];
        pageObj.proxyList = proxyList;
    }
    var proxyRecord = {master:master, view:view, target:proxyTarget};
    proxyList.push(proxyRecord);
};


var TemplateScope = function(pgt, template, pageMaster)
{
    this.protectedInit(pgt);
    this.pageMaster = pageMaster;
    this.template = template;
};

var TemplateScope_proto = new pgt.ExprScope();
TemplateScope.prototype = TemplateScope_proto;
TemplateScope_proto.constructor = TemplateScope;

TemplateScope_proto.resolveName = function(name)
{
    if( this.pageMaster )
    {
        var region = this.pageMaster.getRegion(name);
        if( region !== undef )
            return region;
        var v = pgt.ExprScope.prototype.protectedResolveName.call( this, name );
        if( v !== undef )
            return v;
        return this.pageMaster.template.templateScope.resolveName(name);
    }
    else
        return pgt.ExprScope.prototype.resolveName.call( this, name );
};

var PageMaster = function(pgt, template, cond)
{
    this.pgt = pgt;
    this.template = template;
    this.cond = cond;
    this.regions = [];
    this.regionMap = {};
    this.scope = new TemplateScope(pgt, template, this);
    this.resolveQueue = [];
    this.hasContent = true; // for isEnabled to work correctly
};

var PageMaster_proto = PageMaster.prototype;

PageMaster_proto.getLookupOffset = function()
{
    var context = this.template.context;
    var pageWidth = context.getViewportWidth();
    var pageHeight = context.getViewportHeight();
    var pageArea = pageWidth*pageHeight;
    var fontSize = context.getUnit("em", "height");
    var charArea = fontSize*fontSize; // crude and simple
    var charCountEstimate = pageArea/charArea;
    var uf = 1;
    var p = context.calc(this.props.utilization);
    if( typeof p == "number" )
        uf = p;
    return Math.floor(uf*charCountEstimate);
};

PageMaster_proto.setProperties = function( props )
{
    this.props = props;
};

PageMaster_proto.addRegion = function( name, props, cond )
{
    var region = this.regionMap[name];
    if( region )
    {
        this.pgt.error("region already defined: " + name);
    }
    else
    {
        region = new Region( this.pgt, this, name, props, cond, this.regions.length );
        this.regions.push(region);
        this.regionMap[name] = region;
    }
};

PageMaster_proto.resolve = function()
{
    this.scope.resolve();
    while( this.resolveQueue.length > 0 )
    {
        var expr = this.resolveQueue.pop();
        this.pgt.exprResolve(expr, this.scope);
    } 
};

PageMaster_proto.prepare = function(doc)
{
    this.enabled = true;
    var context = this.template.context;
    var props = this.props;
    
    if( this.cond && !this.cond() )
        enabled = false;
    
    var p = context.calc(props["min-page-width"]);
    if( typeof p == "number" && p < context.getViewportWidth() )
    {
        this.enabled = false;
        return;
    }
        
    p = context.calc(props["min-page-height"]);
    if( typeof p == "number" && p < context.getViewportHeight() )
    {
        this.enabled = false;
        return;
    }
        
    p = context.calc(props.page);
    if( typeof p == "number" && p != this.template.currentPage )
    {
        this.enabled = false;
        return;
    }
    
    p = context.calc(props.enabled);
    if( "" + p == "false" )
    {
        this.enabled = false;
        return;
    }
  
    var i;
    var regions = this.regions;
    for( i = 0 ; i < regions.length ; i++ )
        regions[i].checkContent(doc);
    for( i = 0 ; i < regions.length ; i++ )
    {
        if( !regions[i].prepare(doc) )
        {
            this.enabled = false;
            return;
        }
    }
    
    var snapHeight = context.calc(props["snap-height"]);
    if( snapHeight && typeof snapHeight == "number" && snapHeight > 0 )
        this.snapHeight = snapHeight;
};

PageMaster_proto.layoutPage = function(pageContainer, doc)
{
    var regions = this.regions;
    var pass = 0;
    while(true)
    {
        var done = true;
        for( var i = 0 ; i < regions.length ; i++ )
            if(!regions[i].layoutRegion(pageContainer, doc, pass))
                done = false;
        if( done )
            break;
        doc.setLookupOffset(doc.getCurrentBodyOffset());
        pass++;
    }
    
    if( this.template.showGrid && this.snapHeight )
    {
        var grid = document.createElement("div");
        pageContainer.appendChild(grid);
        for( var y = this.snapHeight ; y < this.pgt.viewportHeight ; y+= this.snapHeight )
        {
            var d = document.createElement("div");
            d.style.position = "absolute";
            d.style.width = "100%";
            d.style.height = "1px";
            d.style.top = (y - 1) + "px";
            d.style.borderBottom = "1px dashed red";
            grid.appendChild(d);
        }
    }
};

PageMaster_proto.getRegion = function(name)
{
    return this.regionMap[name];
};

var Region = function(pgt, pageMaster, name, props, cond, index)
{
    this.pgt = pgt;
    this.pageMaster = pageMaster;
    this.name = name;
    this.cond = cond;
    this.props = props;
    this.index = index;
    this.autoHeightTerm = pgt.createCustomTerm(this.makeAutoHeightTerm());
    this.hasContent = false;
    this.resolvedValue = this; // do be usable in generated expressions
    this.resolveComputed();
};

var Region_proto = Region.prototype;

Region_proto.makeAutoHeightTerm = function()
{
    var self = this;
    return function()
    {
        var h = self.pos.height;
        if( h !== undef )
            return h;
        this.pgt.error( self.name + ".height is not yet resolved" );
        return 0;
    };
};

Region_proto.makeHasContentTerm = function()
{
    var self = this;
    return function() { return self.hasContent; };
};

Region_proto.makeCurrentPage = function()
{
    var template = this.pageMaster.template;
    return function() { return template.currentPage; };    
};

var autoEnabledZero = function(pgt, props, name)
{
    var v = props[name];
    if( v === undef || v == "auto" )
        props[name] = 0;
    else
        props[name] = pgt.exprParse("$enabled?$" + name + ":0", props);
};

Region_proto.resolveComputed = function()
{
    var pgt = this.pgt;
    var props = this.props;
    var virginScope = pgt.createExprScope(); 
    
    props.min = Math.min;
    props.max = Math.max;
    props.floor = Math.floor;
    props.ceil = Math.ceil;
    
    props["pgt-current-page"] = pgt.createCustomTerm(this.makeCurrentPage());
    
    // resolve enabled property
    props["has-content"] = pgt.createCustomTerm(this.makeHasContentTerm());
    if( props.enabled === undef )
        props.enabled = true;
    
    if( props.height === undef )
        props.enabled = pgt.exprParse("$enabled && $has-content", props);
    props["pgt-page-height"] = virginScope.resolveName("page-height");
    var pv = props["min-page-height"];
    if( pv === undef || pv == "auto" )
        props["min-page-height"] = 0;
    else
        props.enabled = pgt.exprParse("$enabled && $min-page-height<=$pgt-page-height", props);
    props["pgt-page-width"] = virginScope.resolveName("page-width");
    pv = props["min-page-width"];
    if( pv === undef || pv == "auto" )
        props["min-page-width"] = 0;
    else
        props.enabled = pgt.exprParse("$enabled && $min-page-width<=$pgt-page-width", props);        
    if( props.page !== undef )
        props.enabled = pgt.exprParse("$enabled && $page==$pgt-current-page", props);        
        
    var arr, i, r;
    var requiredRegions = props["required-regions"];
    if( requiredRegions !== undef )
    {
        arr = (requiredRegions + "").split(/[\s,]+/);
        for( i = 0 ; i < arr.length ; i++ )
        {
            r = this.pageMaster.regionMap[arr[i]];
            if( r === undef )
            {
                pgt.error("Region '" + arr[i] + "' not defined");
                continue;
            }
            props.enabled = pgt.exprParse("$en && $rgn.enabled", {en:props.enabled, rgn:r});
            this.pageMaster.resolveQueue.push(props.enabled);
        }
    }
    
    var conflictingRegions = props["conflicting-regions"];
    if( conflictingRegions !== undef )
    {
        arr = (conflictingRegions + "").split(/[\s,]+/);
        for( i = 0 ; i < arr.length ; i++ )
        {
            r = this.pageMaster.regionMap[arr[i]];
            if( r === undef )
            {
                pgt.error("Region '" + arr[i] + "' not defined");
                continue;
            }
            props.enabled = pgt.exprParse("$en && !($rgn.enabled)", {en:props.enabled, rgn:r});
            this.pageMaster.resolveQueue.push(props.enabled);
        }
    }
    
    var shapeStr = props["outer-shape"];
    if( shapeStr )
        this.outerShapeUnscaled = pgt.createShape(shapeStr);
        
    // resolve horizontal dimensions
    autoEnabledZero(pgt, props, "margin-left");
    autoEnabledZero(pgt, props, "margin-right");
    autoEnabledZero(pgt, props, "padding-left");
    autoEnabledZero(pgt, props, "padding-right");
    autoEnabledZero(pgt, props, "border-left-width");
    autoEnabledZero(pgt, props, "border-right-width");
    if( props.width !== undef )
        props.width = pgt.exprParse("$enabled?$width:0", props);
    
    props["pgt-inset-left"] = pgt.exprParse( "$margin-left+$border-left-width+$padding-left", props );
    props["pgt-inset-right"] = pgt.exprParse( "$margin-right+$border-right-width+$padding-right", props );
    props["pgt-inset-hor"] = pgt.exprParse( "$pgt-inset-left+$pgt-inset-right", props );
    if( props.left !== undef && props.right !== undef && props.width !== undef )
        delete props.width; // overconstraint!
    if( props.left === undef )
    {
        if( props.width === undef || props.right === undef )
            props.left = 0;
        else
            props.left = pgt.exprParse("$pgt-page-width - $right - $width - $pgt-inset-hor", props);
    }
    if( props.right === undef )
    {
        if( props.width === undef )
            props.right = 0;
        else
            props.right = pgt.exprParse("$pgt-page-height - $left - $width - $pgt-inset-hor", props);
    }
    if( props.width === undef )
        props.width = pgt.exprParse("$pgt-page-width - $left - $right - $pgt-inset-hor", props);
    if( props["column-gap"] === undef )
        props["column-gap"] = pgt.exprParse("1em");
    if( props["column-count"] === undef )
    {
        if( props["column-width"] === undef )
            props["column-count"] = 1;
        else
            props["column-count"] = pgt.exprParse("$max(1,$floor(($width+$column-gap)/($column-width+$column-gap)))", props);
    }
    
    // effective value is always computed
    if( props["column-count"] == 1 )
        props["column-width"] = props.width;
    else
        props["column-width"] = pgt.exprParse("($width - $column-gap*($column-count - 1))/$column-count", props);
                
    // resolve vertical dimensions
    autoEnabledZero(pgt, props, "margin-top");
    autoEnabledZero(pgt, props, "margin-bottom");
    autoEnabledZero(pgt, props, "padding-top");
    autoEnabledZero(pgt, props, "padding-bottom");
    autoEnabledZero(pgt, props, "border-top-width");
    autoEnabledZero(pgt, props, "border-bottom-width");
    if( props.height !== undef )
        props.height = pgt.exprParse("$enabled?$height:0", props);
    
    props["pgt-inset-top"] = pgt.exprParse( "$margin-top+$border-top-width+$padding-top", props );
    props["pgt-inset-bottom"] = pgt.exprParse( "$margin-bottom+$border-bottom-width+$padding-bottom", props );
    props["pgt-inset-vert"] = pgt.exprParse( "$pgt-inset-top+$pgt-inset-bottom", props );
    
    if( props.top !== undef && props.bottom !== undef && props.height !== undef )
        delete props.bottom; // overconstraint!
    if( props.height === undef )
    {
        if( props.top === undef || props.bottom === undef )
		{
            props.height = this.autoHeightTerm;
        	props.height = pgt.exprParse("$enabled?$height:0", props);
		}
        else
            props.height = pgt.exprParse( "$pgt-page-height - $bottom - $top - $pgt-inset-vert", props );
    }
    if( props.top === undef )
    {
        if( props.bottom === undef )
            props.top = 0;
        else
            props.top = pgt.exprParse( "$pgt-page-height - $bottom - $height - $pgt-inset-vert", props );
    }
    if( props.bottom === undef )
        props.bottom = pgt.exprParse( "$pgt-page-height - $height - $top - $pgt-inset-vert", props );
        
    // set up -edge values
    props["margin-left-edge"] = props.left;
    props["border-left-edge"] = pgt.exprParse( "$left+$margin-left", props );
    props["padding-left-edge"] = pgt.exprParse( "$margin-left-edge+$border-left-width", props );
    props["left-edge"] = pgt.exprParse( "$padding-left-edge+$padding-left", props );
    props["right-edge"] = pgt.exprParse( "$left-edge+$width", props );
    props["padding-right-edge"] = pgt.exprParse( "$right-edge+$padding-right", props );
    props["border-right-edge"] = pgt.exprParse( "$padding-right-edge+$border-right-width", props );
    props["margin-right-edge"] = pgt.exprParse( "$border-right-edge+$margin-right", props );
    props["margin-top-edge"] = props.top;
    props["border-top-edge"] = pgt.exprParse( "$top+$margin-top", props );
    props["padding-top-edge"] = pgt.exprParse( "$margin-top-edge+$border-top-width", props );
    props["top-edge"] = pgt.exprParse( "$padding-top-edge+$padding-top", props );
    props["bottom-edge"] = pgt.exprParse( "$top-edge+$height", props );
    props["padding-bottom-edge"] = pgt.exprParse( "$bottom-edge+$padding-bottom", props );
    props["border-bottom-edge"] = pgt.exprParse( "$padding-bottom-edge+$border-bottom-width", props );
    props["margin-bottom-edge"] = pgt.exprParse( "$border-bottom-edge+$margin-bottom", props );
    
    props.columns = pgt.exprParse( "$min($1,$column-count)*($column-width+$column-gap) - $column-gap", props, null, true );    
    
    delete props.min;
    delete props.max;
    delete props.floor;
    delete props.ceil;
};

Region_proto.checkContent = function(doc)
{
    this.hasContent = false;
    var flowName = this.props["flow-name"];
    if( !flowName )
        flowName = "body";
    this.hasContent = doc.hasContentInFlow(flowName);
};

Region_proto.prepare = function(doc)
{
    this.pos = {};
    this.outerPos = null;
    var props = this.props;
    var context = this.pageMaster.template.context;
    var required = context.calc(props.required);
    if( "" + required == "true" && props.enabled !== undef )
    {
        var enabled = context.calc(props.enabled);
        if( !enabled )
            return false;
    }
    return true;
};

Region_proto.fillColumnFragment = function(regionContainer, flow, x, y, fragment, modeStyle, fitter)
{
    var pos = fragment.pos;
    var container = document.createElement("div");
    container.style.position = "absolute";
    container.style.left = (pos.x-x) + "px";
    container.style.width = pos.width + "px";
    container.style.top = (pos.y-y) + "px";
    container.style.height = pos.height + "px";
    container.style.overflow = "hidden";
    regionContainer.appendChild(container);
    
    if( this.pageMaster.template.showFragments )
    {
        container.style.outline = "1px dotted green";
        container.style.backgroundColor = "yellow";
    }
    
    var elemFunc;
    if( modeStyle )
        elemFunc = modeStyle.getApplyViewStylesFunc();
    else
        elemFunc = this.pgt.applyViewStyles;
    if( fitter )
        fitter.beginFragment( container, pos.width, pos.height, fragment.floats, modeStyle );
    var h = flow.flowInto(container, null, fragment.floats, elemFunc, fitter, !fragment.skip);
    if( fitter )
        fitter.endFragment(pos.width, pos.height);
    return h;
};

var addColumnRule = function( regionContainer, x, y, geom, props, fragment )
{
    var gap = geom.columnGap;
    var width = geom.columnRuleWidth;
    var pos = fragment.pos;
    var container = document.createElement("div");
    container.style.position = "absolute";
    container.style.left = (pos.x-x-(gap+width)/2) + "px";
    container.style.width = "1px";
    container.style.top = (pos.y-y) + "px";
    container.style.height = pos.height + "px";
    container.style.overflow = "hidden";
    container.style.borderLeftWidth = width + "px";
    container.style.borderLeftColor = props["column-rule-color"];
    container.style.borderLeftStyle = props["column-rule-style"];
    regionContainer.appendChild(container);
};

Region_proto.doHorizontalGeometry = function(geom)
{
    var pos = this.pos;
    var props = this.props;
    var context = this.pageMaster.template.context;
    
    geom.left = context.calc(props.left);
    geom.marginLeft = context.calc(props["margin-left"]);
    geom.marginRight = context.calc(props["margin-right"]);
    geom.paddingLeft = context.calc(props["padding-left"]);
    geom.paddingRight = context.calc(props["padding-right"]);
    geom.borderLeft = context.calc(props["border-left-width"]);
    geom.borderRight = context.calc(props["border-right-width"]);
    geom.width = context.calc(props.width);
    geom.columnGap = context.calc(props["column-gap"]);
    geom.columnCount = context.calc(props["column-count"]);
    geom.columnWidth = context.calc(props["column-width"]);
    geom.columnRuleWidth = context.calc(props["column-rule-width"]);
    geom.outerWidth = geom.width + context.calc(props["pgt-inset-hor"]);
    
    return geom.left + geom.marginLeft + geom.borderLeft + geom.paddingLeft;    
}; 

Region_proto.doVerticalGeometry = function(geom, only)
{
    var pos = this.pos;
    var props = this.props;
    var pageMaster = this.pageMaster;
    var context = pageMaster.template.context;
    
    var snapHeight = context.calc(props["snap-height"]);
    if( !snapHeight )
        snapHeight = pageMaster.snapHeight;
    if( typeof snapHeight == "number" && snapHeight > 0 )
        geom.snapHeight = snapHeight;
        
    if( !only || only == "top" )
    {
        geom.top = context.calc(props.top);
        geom.marginTop = context.calc(props["margin-top"]);
        geom.borderTop = context.calc(props["border-top-width"]);
        geom.paddingTop = context.calc(props["padding-top"]);
        if( snapHeight )
        {
            var topEdge = geom.top + geom.marginTop + geom.borderTop + geom.paddingTop;
            var snapOffset = Math.ceil(topEdge/snapHeight)*snapHeight - topEdge;
            geom.top += snapOffset;
        }
    }
    if( !only || only == "bottom" )
    {
        geom.borderBottom = context.calc(props["border-bottom-width"]);
        geom.marginBottom = context.calc(props["margin-bottom"]);
        geom.paddingBottom = context.calc(props["padding-bottom"]);
        geom.height = context.calc(props.height);
        geom.outerHeight = geom.height + context.calc(props["pgt-inset-vert"]);
    }
    
    return geom.top + geom.marginTop + geom.borderTop + geom.paddingTop;
}; 

Region_proto.setHorizontalGeometry = function( style, geom )
{
    style.left = geom.left + "px";
    style.marginLeft = geom.marginLeft + "px";
    style.marginRight = geom.marginRight + "px";
    style.borderLeftWidth = geom.borderLeft + "px";
    style.borderRightWidth = geom.borderRight + "px";
    style.paddingLeft = geom.paddingLeft + "px";
    style.paddingRight = geom.paddingRight + "px";
    style.width = geom.width + "px";
};

Region_proto.setVerticalGeometry = function( style, geom )
{
    style.top = geom.top + "px";
    style.marginTop = geom.marginTop + "px";
    style.marginBottom = geom.marginBottom + "px";
    style.borderTopWidth = geom.borderTop + "px";
    style.borderBottomWidth = geom.borderBottom + "px";
    style.paddingTop = geom.paddingTop + "px";
    style.paddingBottom = geom.paddingBottom + "px";
    style.height = geom.height + "px";
};

Region_proto.transferBordersAndBackground = function( style, props )
{
    style.borderTopStyle = props["border-top-style"];
    style.borderBottomStyle = props["border-bottom-style"];
    style.borderLeftStyle = props["border-left-style"];
    style.borderRightStyle = props["border-right-style"];
    style.borderTopColor = props["border-top-color"];
    style.borderBottomColor = props["border-bottom-color"];
    style.borderLeftColor = props["border-left-color"];
    style.borderRightColor = props["border-right-color"];
    style.backgroundColor = props["background-color"];
    style.backgroundImage = props["background-image"];
    style.backgroundRepeat = props["background-repeat"];
    style.backgroundAttachment = props["background-attachment"];
    style.backgroundPosition = props["background-position"];
    
    var context = this.pageMaster.template.context;
    style.zIndex = context.calc(props["z-index"]);
    style.opacity = context.calc(props.opacity);
};

var snapRectIfNeeded = function(rect, snapHeight)
{
    if( !snapHeight )
        return rect;
    var top = rect.y;
    var bottom = top + rect.height;
    top = Math.floor(top/snapHeight)*snapHeight;
    bottom = Math.ceil(bottom/snapHeight)*snapHeight;
    return {x:rect.x, y:top, width:rect.width, height:bottom-top};
};

Region_proto.doLayout = function(pageContainer, doc, pass)
{
    var pgt = this.pgt;
    var props = this.props;
    var template = this.pageMaster.template;
    var context = template.context;
    var document = pgt.document;
    
    var flowName = props["flow-name"];
    if( !flowName )
        flowName = "body";
        
    var fitter = template.fitters[flowName];

    var geom = this.geom;
    var pos = this.pos;
    
    var y = geom.regionY;
    var slidingY = geom.slidingY;
    var autoHeight = geom.autoHeight;
    var height = geom.regionHeight;
    
    var computedHeight = 0;
    var regionContainer = document.createElement("div");
    this.regionContainer = regionContainer;
    pageContainer.appendChild(regionContainer);
    var style = regionContainer.style;
    style.position = "absolute";
    style.overflow = "hidden";
    this.transferBordersAndBackground(style, props);
    this.setHorizontalGeometry(style, geom);
    if( !autoHeight )
        this.setVerticalGeometry(style, geom);
    else
    {
        style.top = y + "px";
        style.height = height + "px";
    }
    regionContainer.setAttribute("region-name", this.name);
        
    var flow;
    if( flowName )
    {
        flow = doc.getFlow(flowName);
        if( flow )
        {
            var overflowControl = props["overflow-control"]; // auto, skip, place
            if( !overflowControl )
                overflowControl = "auto";
            var synth = context.calc(props["float-rendering"]) == "synthetic";
            var modeName = context.calc(props.mode);
            var modeStyle = (modeName ? template.modes[modeName] : null);
            var width = geom.columnWidth;
            flow.beginRegion(geom.width, height, width);
            var px = pos.x - geom.paddingLeft;
            var py = y - (slidingY?0:geom.paddingTop);
            var i;
            for( var column = 0 ; column < geom.columnCount ; column++ )
            {
                var x = pos.x + column*(width + geom.columnGap);
                var overlaps = null;
                if( !slidingY )
                {
                    overlaps = [];
                    for( i = 0 ; i < this.index ; i++ )
                    {
                        var region = this.pageMaster.regions[i];
                        var outerRect = region.outerPos;
                        if( outerRect )
                        {
                            if( region.outerShape )
                            {
                                var gridStep = geom.snapHeight ? geom.snapHeight : pgt.units.em*3;
                                var overlapRects = region.outerShape.toRects(y, gridStep);
                                if( overlapRects.length > 0 )
                                    overlaps.push.apply(overlaps, overlapRects);
                            }
                            else
                                overlaps.push(snapRectIfNeeded(outerRect, geom.snapHeight));
                        }
                    }
                }
                var fragments = pgt.makeColumnFragments( x, y, width, height, overlaps, synth, overflowControl );
                for( i = 0 ; i < fragments.length ; i++ )
                {
                    var fragment = fragments[i];
                    var h = this.fillColumnFragment( regionContainer, flow, px, py, fragment, modeStyle, fitter );
                    if( column > 0 && geom.columnRuleWidth )
                        addColumnRule( regionContainer, px, py, geom, props, fragment );
                    h += fragment.pos.y - py;
                    if( h > computedHeight )
                        computedHeight = h;
                }
            }
            flow.endRegion();
        }
    }
    return computedHeight;
};

Region_proto.doPlacement = function(pageContainer, doc, delayed)
{
    var pgt = this.pgt;
    var props = this.props;
    var template = this.pageMaster.template;
    var context = template.context;
    var document = pgt.document;
    
    var flowName = this.props["flow-name"];
    if( !flowName )
        flowName = "body";
            
    var fitter = template.fitters[flowName];
    
    var geom = {};
    this.geom = geom;
    var pos = this.pos;

    pos.x = this.doHorizontalGeometry(geom);
    pos.width = geom.width;
    
    var y, height, d;
    var autoHeight = pgt.exprDependsOn(props.height, this.autoHeightTerm);
    if( autoHeight && delayed )
    {
        autoHeight = false;
        pos.height = 0;
        pgt.error("height:auto with content-selection:late");
    }
    var slidingY = false;
    if( autoHeight )
    {
        slidingY = pgt.exprDependsOn(props["top-edge"], this.autoHeightTerm);
        height = 100*context.getUnit("%", "height");
        if( slidingY )
            y = 0;
        else
        {
            y = this.doVerticalGeometry(geom, "top");
            height -= y;
            if( !pgt.exprDependsOn(props["pgt-inset-bottom"], this.autoHeightTerm) )
            {
                d = context.calc(props["pgt-inset-bottom"]);
                height -= d;
            }
            if( !pgt.exprDependsOn(props.bottom, this.autoHeightTerm) )
            {
                d = context.calc(props.bottom);
                height -= d;
            }
        }
        if( fitter )
        {
            pos.height = height;
            y = this.doVerticalGeometry(geom);
            pos.y = y;
            autoHeight = false;
        }
    }
    else
    {
        y = this.doVerticalGeometry(geom);
        height = geom.height;
        pos.y = y;
        pos.height = height;
    }

    geom.regionY = y;
    geom.slidingY = slidingY;
    geom.regionHeight = height;
    geom.autoHeight = autoHeight;
    
    var computedHeight = 0;
    if( !delayed )
        computedHeight = this.doLayout(pageContainer, doc, 0 );         
    
    if( autoHeight )
    {
        pos.height = computedHeight;
        pos.y = context.calc(props.top);
        this.doVerticalGeometry(geom, (slidingY?null:"bottom"));
        this.setVerticalGeometry(this.regionContainer.style, geom);
     }
     
     // autoHeight regions does't produce anything unless they have non-zero height
     if( autoHeight && geom.height === 0 )
        pageContainer.removeChild(this.regionContainer);
     else
     {
        this.overlap = context.calc(props["region-overlap"]);
        if( this.overlap != "overlap" )
        {
            this.outerPos = {x:geom.left, y:geom.top, width:geom.outerWidth, height:geom.outerHeight, overlap:this.overlap};
            if( this.outerShapeUnscaled && this.overlap != "clear" )
                this.outerShape = this.outerShapeUnscaled.scaleToRect(this.outerPos);
        }
        
        if( template.showRegions )
            this.regionContainer.style.outline = "1px solid blue";
        
        if( template.showRegionMargins )
        {
            var d = document.createElement("div");
            d.style.position = "absolute";
            d.style.top = geom.top + "px";
            d.style.left = geom.left + "px";
            d.style.width = geom.outerWidth + "px";
            d.style.height = geom.outerHeight + "px";
            d.style.outline = "1px dotted blue";
            pageContainer.appendChild(d);
        }
     }
};

Region_proto.layoutRegion = function(pageContainer, doc, pass)
{   
    var pgt = this.pgt;
    var props = this.props;
    var template = this.pageMaster.template;
    var context = template.context;
    
    if( props.enabled !== undef )
    {
        var enabled = context.calc(props.enabled);
        if( !enabled )
            return true;     
    }
    
    var contentSelection = context.calc(props["content-selection"]);
    var targetPass = (contentSelection == "late" ? 1 : 0);
    
    if( pass == 0 )
        this.doPlacement(pageContainer, doc, targetPass > 0);
 
    if( pass < targetPass )
        return false; // cannot be done yet
    if( pass > targetPass )
        return true; // already done
 
    if( pass != 0 )
        this.doLayout(pageContainer, doc, pass);
    
    return true;
};

Region_proto.pgtResolveAccess = function(name) {
    var val = this.props[name];
    if( val !== undef )
        return val;
    this.pgt.error( this.name + "." + name + " not defined" );
    return null;
};

})(pgt);