// Page Template Library
// core utilities

// begin private scope
var pgt = (function(window, undef) {

var PGT = function(param, param2)
{
    if( param )
    {
        this.protectedInit(param, param2);
    }
};

var pgt = PGT.prototype;

pgt.PGT = PGT;

pgt.protectedInit = function(param, param2)
{
    if( param.nodeType == 1 )
    {
        this.body = param;
        this.viewport = this.body;
        this.rootEventTarget = this.body;
        this.document = this.body.ownerDocument;
        this.window = this.document.defaultView;
    }
    else if( param.nodeType == 9 )
    {
        this.document = param;
        this.body = this.document.body;
        this.rootEventTarget = this.document;
        this.window = this.document.defaultView;
    }
    else if( param.document )
    {
        this.window = param; 
        this.document = this.window.document;
        this.rootEventTarget = this.document;
        this.body = this.document.body;
    }
    if( param2 && param2.nodeType == 9 )
    {
        this.sourceDocument = param2;
        var list = param2.getElementsByTagName("body");
        if( list.length == 0 )
            this.error("no body");
        else
            this.source = list.item(0);
    }
    else if( param2 && param2.nodeType == 1 )
    {
        this.source = param2;
        this.sourceDocument = this.source.ownerDocument;
    }
    else
        this.sourceDocument = this.document;
    this.privateInit();
};

pgt.privateInit = function()
{
    var pid = {};
    this.privateInstanceData = pid;
    // default values
    this.units = {em:16, ex:8, pt:4/3.0, rpx:1};
    this.viewportWidth = 500;
    this.viewportHeight = 500;
    this.loaded = false;
    this.viewContainer = null;
    this.originalContainer = null;
    this.needEvents = true;
    this.needUIEvents = false;
    this.baseURL = null;
    pid.tasks = [];
    pid.timers = {};
    pid.taskQueueRunning = false;
    pid.taskTimeout = null;
    pid.activeTask = null;
    pid.taskState = null;
    pid.pgt = this;
};

var modules = {};

pgt.window = window; // default window
pgt.document = window.document; // default document (to create nodes)
pgt.rootEventTarget = pgt.document; // default root event target
pgt.sourceDocument = pgt.document;
pgt.body = null; // root DOM container
pgt.viewport = null;
pgt.privateInit(); // initialize default instance

var ua = navigator.userAgent;

var isIThing = ua.indexOf("iPad;") > 0 || ua.indexOf("iPhone;") > 0;

var isIE = ua.indexOf("; MSIE") > 0;

pgt.requires = function(name)
{
    if( !modules[name] )
        alert(name + " required, perhaps " + name + ".js is missing?" );
};

pgt.provides = function(name)
{
    modules[name] = name;
};

pgt.isWebKit = function()
{
    var pid = this.privateInstanceData;
    if( pid.isWebKit === undef )
        pid.isWebKit = this.document.body.style.WebkitTransition !== undef;
    return pid.isWebKit;
};

pgt.isIThing = function()
{
    return isIThing;
};

pgt.isIE = function()
{
    return isIE;
};

pgt.log = function(msg)
{
    if( console && console.log )
        console.log(msg);
};

pgt.trace = function(msg)
{
    this.log("TRACE: " + msg);
};

pgt.error = function(msg)
{
    this.log("ERROR: " + msg);
    throw msg;
};

pgt.startTimer = function(name)
{
    var pid = this.privateInstanceData;
    pid.timers[name] = (new Date()).valueOf();
};

pgt.stopTimer = function(name)
{
    var pid = this.privateInstanceData;
    this.log( "TIMER: " + name + ": " + ((new Date()).valueOf() - pid.timers[name]) );
};

pgt.status = function(id, msg)
{
    var pid = this.privateInstanceData;
    var statusMap = pid.statusMap;
    if( !statusMap )
    {
        statusMap = {};
        pid.statusMap = statusMap;
    }
    statusMap[id] = msg;
    showStatus(pid, id, msg);
    this.log( "STATUS: " + id + ": " + msg);
};

var showStatus = function(pid, id, msg)
{
    if( pid.statusWindow )
    {
        var doc = pid.statusWindow.document;
        if( doc )
        {
            var e = doc.getElementById(id);
            if( e == null )
            {
                e = doc.createElement("div");
                e.setAttribute("id", id);
                e.appendChild( doc.createTextNode(id + ": ") );
                e.appendChild( doc.createElement("span") );
                doc.body.appendChild(e);
            }
            e.lastChild.textContent = msg;
        }
    }
};

var createStatusWindow = function(pgt)
{
    var pid = pgt.privateInstanceData;
    if( pid.statusWindow && pid.statusWindow.document )
        return;
    pid.statusWindow = window.open("", "status",
        "status=0,toolbar=0,location=0,menubar=0,directories=0,resizable=1,width=500,height=400");
    pid.statusWindow.document.write("<html><head><title>Status</title></head><body></body></html>"); 
    var statusMap = pid.statusMap;
    if( statusMap )
    {
        for( var id in statusMap )
            showStatus( pid, id, statusMap[id] );
    }
};

pgt.resolveURL = function( relURL, baseURL )
{
    if( !baseURL )
    {
        baseURL = this.baseURL;
        if( !baseURL )
            return relURL;
    }
    if( relURL.match(/^\w\w+:/) || relURL.indexOf("/") == 0 )
        return relURL;
    if( relURL == "." )
        relURL = "";
    var lastSlash = baseURL.lastIndexOf("/");
    if( lastSlash < 0 )
        return relURL;
    var url = baseURL.substr(0,lastSlash+1) + relURL;
    while(true)
    {
    	var i = url.indexOf("/../");
    	if( i <= 0 )
    		break;
    	var j = url.lastIndexOf("/",i-1);
    	if( j <= 0 )
    		break;
    	url = url.substr(0,j) + url.substr(i+3);
    }
    return url;
};


pgt.enterPrintingMode = function( width, height, margin )
{
    if( !width )
        width = 612 * this.units.pt;
    if( !height )
        height = 792 * this.units.pt;
    if( !margin )
        margin = 36 * this.units.pt;
    this.privateInstanceData.printData = {
        width: width-2*margin,
        height: height-2*margin,
        margin: margin
    };
    this.onresize();
};

pgt.exitPrintingMode = function()
{
    delete pid.privateInstanceData.printData;
    this.onresize();
};

pgt.isPrinting = function()
{
    return !!this.privateInstanceData.printData;
};

pgt.getPrintMargin = function()
{
    return this.privateInstanceData.printData.margin;
};

pgt.probeUnits = function(envElem)
{
	var units = this.units;
	
    var document = this.document;
    var probe = document.createElement("div");
    probe.style.position = "absolute";
    probe.style.lineHeight = "1";
    probe.style.visibility = "hidden";
    if( !envElem )
        envElem = this.body;
    envElem.appendChild(probe);
    var span = document.createElement("span");
    probe.appendChild(span);
    span.appendChild(document.createTextNode("M"));
    
    probe.style.fontSize = "10em";
    var r = probe.getBoundingClientRect();
    if( r.height > 20 )
        units._em = r.height/10;
        
    probe.style.fontSize = "144pt";
    r = probe.getBoundingClientRect();
    if( r.height > 5 )
        units._pt = r.height/144.0;
        
    probe.style.fontSize = "20ex";
    r = probe.getBoundingClientRect();
    if( r.height > 5 )
        units._ex = r.height/20.0;
        
    probe.style.fontSize = "3em";
    var h0 = span.getBoundingClientRect().height;
    probe.style.WebkitTransform = "scale(5)";
    probe.style.MozTransform = "scale(5)";
    probe.style.OTransform = "scale(5)";
    var h1 = span.getBoundingClientRect().height;
    this.unscaledBoundingClientRect = h1 > 2*h0;
        
    envElem.removeChild(probe);
};

pgt.updateUnits = function(envElem, textScale, globalScale)
{
	var units = this.units;
	
	if( !this.privateInstanceData.probed )
	{
		this.privateInstanceData.probed = true;
		this.probeUnits(envElem);
	}
	
	globalScale = globalScale || 1;
	textScale = globalScale * (textScale || 1);
	
	units.em = units._em * textScale;
	units.ex = units._ex * textScale;
	units.pt = units._pt * globalScale;
	units.rpx = globalScale;
};

pgt.getBoundingClientRect = function(elem)
{
    var rect = elem.getBoundingClientRect();
    var scale = this.units.rpx;
    if( scale == 1 || !this.unscaledBoundingClientRect )
        return rect;
    return {
        top: scale*rect.top,
        bottom: scale*rect.bottom,
        left: scale*rect.left,
        right: scale*rect.right,
        width: scale*rect.width,
        height: scale*rect.height
    };
};

pgt.updateViewportSize = function(globalScale)
{
	var printData = this.privateInstanceData.printData;
    if( printData )
    {
        this.viewportWidth = printData.width;
        this.viewportHeight = printData.height;       
    }
    else if( this.viewport )
    {
        var rect = this.getBoundingClientRect(this.viewport);
        this.viewportWidth = rect.width;
        this.viewportHeight = rect.height;
    }
    else
    {
		this.viewportWidth = this.window.innerWidth;
        this.viewportHeight = this.window.innerHeight;
    }
};

pgt.getViewportWidth = function()
{
	return this.viewportWidth * this.units.rpx;
};

pgt.getViewportHeight = function()
{
	return this.viewportHeight * this.units.rpx;
};

var nextTaskLater = function(pgt)
{
    var pid = pgt.privateInstanceData;
    pid.taskQueueRunning = true;
    pid.taskTimeout = setTimeout(nextTask, 1, pgt);
};

var nextTask = function(pgt)
{
    var pid = pgt.privateInstanceData;
    pid.taskQueueRunning = true;
    if( pid.taskTimeout )
    {
        clearTimeout(pid.taskTimeout);
        pid.taskTimeout = null;
    }
    if( pid.tasks.length === 0 )
    {
        pid.taskQueueRunning = false;
        return;
    }
    pid.activeTask = pid.tasks.shift();
    pid.taskState = "started";
    pid.activeTask();
    if( pid.taskState != "done" )
        pid.taskState = "running";
};

pgt.scheduleTask = function(task)
{
    var pid = this.privateInstanceData;
    pid.tasks.push(task);
    if( this.loaded && !pid.taskQueueRunning )
        nextTaskLater(this);
};

pgt.scheduleCall = function(func)
{
    var pgt = this;
    var task = function() { func(); pgt.taskDone(); };
    pgt.scheduleTask(task);
};

pgt.taskDone = function()
{
    var pid = this.privateInstanceData;
    pid.taskState = "done";
    nextTaskLater(this);
};

var download = function(pgt, url, handler, field)
{
    var req = new XMLHttpRequest();
    url = pgt.resolveURL(url);
    var taskEnd = function()
    {
        if( req.readyState == 4 )
        {
        	if( field == "responseXML" && !req[field].URL )
        		req[field].URL = url; // was unset in Mozilla
            handler(req[field], url);
            pgt.taskDone();
        }
    };
    var taskStart = function()
    {
        var urlToUse = url;
        req.open("GET", urlToUse, true);
        req.onreadystatechange = taskEnd;
        req.send(null);  
    };
    pgt.scheduleTask(taskStart);
};

pgt.downloadText = function(url, handler)
{
    download(this, url, handler, "responseText");
};

pgt.downloadXML = function(url, handler)
{
    download(this, url, handler, "responseXML");
};

pgt.patterns = {
	cssLengthPart: /^\s*(\d+(\.\d*)?)(%|em|ex|in|pt|px|mm|cm|rem|rpx)?/,
	cssLength: /^\s*(\d+(\.\d*)?)(%|em|ex|in|pt|px|mm|cm|rem|rpx)?\s*$/,
	idFragment: /^#[-.\w\u0080-\uFFFF]+$/		
};

pgt.ns = {
	xhtml: "http://www.w3.org/1999/xhtml"
};

pgt.trigger = function(target, name, param )
{
};

pgt.createView = function(elem, recursive)
{
    var e;
    if( !recursive && elem.nodeName == "body" )
        return this.document.createElement("div");
    if( this.document == this.sourceDocument )
        e = elem.cloneNode(recursive);
    else
        e = this.document.importNode(elem, recursive);
    e.removeAttribute("id");
    return e;
};

pgt.addView = function(elem, view, recursive)
{
};

pgt.clearViews = function(elem)
{
};

pgt.startTaskQueue = function()
{
    if( this.privateInstanceData.loaded )
        return;
    this.privateInstanceData.loaded = true;
    
    if( !this.body )
        this.body = this.window.document.body;
    
    this.loaded = true;
    nextTask(this);   
};

pgt.startTaskQueueOnLoad = function()
{
	pgt.window.addEventListener( "load", function() { pgt.startTaskQueue(); }, false );
};

return pgt;
})(window);
