// Page Template Library
// driver script implement HTML stack in magazine viewer

// begin private scope
(function(pgt) {

if( !pgt )
    alert("pgt-util.js is required");
pgt.provides("pgt-tpage");
pgt.requires("pgt-style");
pgt.requires("pgt-flow");
pgt.requires("pgt-expr");
pgt.requires("pgt-dstyle");
pgt.requires("pgt-fit");
pgt.requires("pgt-page");
pgt.requires("pgt-templ");
	
var StackDriver = function(pgt, viewport, templateBaseURL)
{
	pgt = new pgt.PGT(viewport); // private PGT environment - check if this is really necessary?
	this.pgt = pgt;
	this.pgt.needEvents = false; //prevent custom hyperlink handling
	this.baseURL = templateBaseURL;
	this.customizePGT();
    this.template = pgt.createTemplate();
	this.transition = "none";
	this.stacking = "vertical";
	this.viewport = this.pgt.viewport; //shortcut refence to viewport
};

pgt.createStackDriver = function(viewport, baseURL)
{
    return new StackDriver(this, viewport, baseURL);
};

var StackDriver_proto = StackDriver.prototype;

StackDriver_proto.setViewportSize = function(width, height)
{
	this.viewportWidth = width;
	this.viewportHeight = height;
	this.pgt.updateViewportSize();
};

StackDriver_proto.setSourceDocument = function(srcDoc)
{
	var pgt = this.pgt;
    pgt.sourceDocument = srcDoc;
    pgt.baseURL = srcDoc.URL;
    var list = srcDoc.getElementsByTagName("body");
    if( list.length == 0 )
        pgt.error("no body");
    else
        pgt.source = list.item(0);

	this.doc = pgt.createDoc();
	this.doc.addElement(pgt.source);
    this.pagedView = pgt.createPagedView();
	this.pagedView.setPageTransition(this.transition, this.stacking);
	this.pagedView.setPageProducer(this.template.createPageProducer(this.doc));    
};

StackDriver_proto.enableKeyboardNavigation = function(){
	this.transition = "verticalSlide";
	this.stacking = "single";
	
	//prevent scrollbars
	var body = document.body;
	body.style.overflowY = "hidden";
	body.style.overflowX = "hidden";
	
	this.viewport.focus();
	this.viewport.addEventListener("keydown", function(e){
		switch (e.keyCode){
			//right arrow
			case 39: 
				driver.pagedView.nextPage();
				e.preventDefault();
			break;
			
			//left arrow
			case 37:
				driver.pagedView.previousPage();
				e.preventDefault();
			break;
			
			//up arrow
			case 38:
				driver.pagedView.previousPage();
				e.preventDefault();
			break;
			
			//down arrow
			case 40:
				driver.pagedView.nextPage();
				e.preventDefault();
			break;
		}
	}, false);
}

StackDriver_proto.setTemplate = function(templateTxt){
	this.template.parseStylesheet(templateTxt, null, this.baseURL);
	this.pgt.updateUnits();
	this.pgt.startTaskQueue();
	this.template.resolve();
	this.setViewportSize(window.innerWidth -20, window.innerHeight); // scrollbar allowance
}

var fixSRC = function(e)
{
	var src = e.getAttribute("src");
	src = this.resolveURL(src);
	e.setAttribute("src", src);
};

StackDriver_proto.createView = function(elem, recursive)
{
    var e;
    var nodeName = elem.nodeName;
    if( !recursive && (nodeName == "body" || nodeName == "BODY") )
    {
		e = this.document.createElement("div");
		e.className = elem.className;
		return e;
	}
    if( this.document == this.sourceDocument )
        e = elem.cloneNode(recursive);
    else
		e = this.document.importNode(elem, recursive);
	if( recursive )
	{
		if( e.nodeType == 1 )
		{
			var list = e.getElementsByTagName("img");
			var len = list.length;
			for( var i = 0 ; i < len ; i++ )
				this.fixSRC(list.item(i));
		}
	}
	if( nodeName == "img" || nodeName == "IMG" )
		this.fixSRC(e);
		
	if (!this.IdCache){
		this.IdCache = [];
	}
	
	//Check if this ID has already been assigned
	//Using static cache because getElementById works only after DOM insertion
	if (e.id)
	{
		if (this.IdCache.indexOf(e.id) > -1)
		{
			//id has already been used once; don't apply it
			e.removeAttribute("id");
		}
		else
		{
			//first encounter of the ID; store it for future checking
			this.IdCache.push(e.id);
		}
	}

    return e;
};

var updateViewportSize = function(globalScale)
{
	this.viewportWidth = this.driver.viewportWidth;
    this.viewportHeight = this.driver.viewportHeight;
};

StackDriver_proto.customizePGT = function()
{
	var pgt = this.pgt;
	pgt.driver = this;
	pgt.fixSRC = fixSRC;
	pgt.createView = this.createView;
	pgt.updateViewportSize = updateViewportSize;
};

StackDriver_proto.nextPage = function()
{
	if( this.pagedView )
		return this.pagedView.nextPage();
};

})(pgt);

